/****************************************************************************
** Meta object code from reading C++ file 'downloader.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/downloader.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'downloader.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Downloader_t {
    QByteArrayData data[18];
    char stringdata0[160];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Downloader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Downloader_t qt_meta_stringdata_Downloader = {
    {
QT_MOC_LITERAL(0, 0, 10), // "Downloader"
QT_MOC_LITERAL(1, 11, 13), // "activeChanged"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 13), // "loadedChanged"
QT_MOC_LITERAL(4, 40, 12), // "totalChanged"
QT_MOC_LITERAL(5, 53, 19), // "proxyAddressChanged"
QT_MOC_LITERAL(6, 73, 6), // "cancel"
QT_MOC_LITERAL(7, 80, 3), // "get"
QT_MOC_LITERAL(8, 84, 3), // "url"
QT_MOC_LITERAL(9, 88, 4), // "hash"
QT_MOC_LITERAL(10, 93, 8), // "QJSValue"
QT_MOC_LITERAL(11, 102, 8), // "callback"
QT_MOC_LITERAL(12, 111, 10), // "saveToFile"
QT_MOC_LITERAL(13, 122, 4), // "path"
QT_MOC_LITERAL(14, 127, 6), // "active"
QT_MOC_LITERAL(15, 134, 6), // "loaded"
QT_MOC_LITERAL(16, 141, 5), // "total"
QT_MOC_LITERAL(17, 147, 12) // "proxyAddress"

    },
    "Downloader\0activeChanged\0\0loadedChanged\0"
    "totalChanged\0proxyAddressChanged\0"
    "cancel\0get\0url\0hash\0QJSValue\0callback\0"
    "saveToFile\0path\0active\0loaded\0total\0"
    "proxyAddress"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Downloader[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       4,   64, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,
       4,    0,   51,    2, 0x06 /* Public */,
       5,    0,   52,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       6,    0,   53,    2, 0x02 /* Public */,
       7,    3,   54,    2, 0x02 /* Public */,
      12,    1,   61,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, 0x80000000 | 10,    8,    9,   11,
    QMetaType::Bool, QMetaType::QString,   13,

 // properties: name, type, flags
      14, QMetaType::Bool, 0x00495001,
      15, QMetaType::ULongLong, 0x00495001,
      16, QMetaType::ULongLong, 0x00495001,
      17, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

       0        // eod
};

void Downloader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Downloader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeChanged(); break;
        case 1: _t->loadedChanged(); break;
        case 2: _t->totalChanged(); break;
        case 3: _t->proxyAddressChanged(); break;
        case 4: _t->cancel(); break;
        case 5: { bool _r = _t->get((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QJSValue(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->saveToFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Downloader::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Downloader::activeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Downloader::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Downloader::loadedChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Downloader::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Downloader::totalChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Downloader::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Downloader::proxyAddressChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Downloader *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        case 1: *reinterpret_cast< quint64*>(_v) = _t->loaded(); break;
        case 2: *reinterpret_cast< quint64*>(_v) = _t->total(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->proxyAddress(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Downloader *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setProxyAddress(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Downloader::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Downloader.data,
    qt_meta_data_Downloader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Downloader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Downloader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Downloader.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Downloader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Downloader::activeChanged()const
{
    QMetaObject::activate(const_cast< Downloader *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Downloader::loadedChanged()const
{
    QMetaObject::activate(const_cast< Downloader *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Downloader::totalChanged()const
{
    QMetaObject::activate(const_cast< Downloader *>(this), &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Downloader::proxyAddressChanged()const
{
    QMetaObject::activate(const_cast< Downloader *>(this), &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
