// Copyright (c) 2014-2024, The Monero Project
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
//    conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
//    of conditions and the following disclaimer in the documentation and/or other
//    materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors may be
//    used to endorse or promote products derived from this software without specific
//    prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef OSHELPER_H
#define OSHELPER_H

#include <QList>
#include <QObject>
#include <QString>
/**
 * @brief The OSHelper class - exports to QML some OS-related functions
 */
class OSHelper : public QObject
{
    Q_OBJECT
    Q_PROPERTY(bool installed READ installed CONSTANT);

public:
    explicit OSHelper(QObject *parent = 0);

    Q_INVOKABLE void createDesktopEntry() const;
    Q_INVOKABLE QString downloadLocation() const;
    Q_INVOKABLE QList<QString> grabQrCodesFromScreen() const;
    Q_INVOKABLE bool openFile(const QString &filePath) const;
    Q_INVOKABLE bool openContainingFolder(const QString &filePath) const;
    Q_INVOKABLE QString openSaveFileDialog(const QString &title, const QString &folder, const QString &filename) const;
    Q_INVOKABLE QString temporaryFilename() const;
    Q_INVOKABLE QString temporaryPath() const;
    Q_INVOKABLE QString randomPassword(int numBytes = 32) const;
    Q_INVOKABLE bool removeTemporaryWallet(const QString &walletName) const;
    Q_INVOKABLE bool isCapsLock() const;
    Q_INVOKABLE quint8 getNetworkTypeFromFile(const QString &keysPath) const;
    Q_INVOKABLE void openSeedTemplate() const;

    static std::pair<quint8, QString> getNetworkTypeAndAddressFromFile(const QString &wallet);
private:
    bool installed() const;

signals:

public slots:
};

#endif // OSHELPER_H
