/****************************************************************************
** Meta object code from reading C++ file 'WalletManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/libwalletqt/WalletManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'WalletManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_WalletManager_t {
    QByteArrayData data[119];
    char stringdata0[1541];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_WalletManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_WalletManager_t qt_meta_stringdata_WalletManager = {
    {
QT_MOC_LITERAL(0, 0, 13), // "WalletManager"
QT_MOC_LITERAL(1, 14, 12), // "walletOpened"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 7), // "Wallet*"
QT_MOC_LITERAL(4, 36, 6), // "wallet"
QT_MOC_LITERAL(5, 43, 13), // "walletCreated"
QT_MOC_LITERAL(6, 57, 22), // "walletPassphraseNeeded"
QT_MOC_LITERAL(7, 80, 8), // "onDevice"
QT_MOC_LITERAL(8, 89, 19), // "deviceButtonRequest"
QT_MOC_LITERAL(9, 109, 10), // "buttonCode"
QT_MOC_LITERAL(10, 120, 19), // "deviceButtonPressed"
QT_MOC_LITERAL(11, 140, 20), // "checkUpdatesComplete"
QT_MOC_LITERAL(12, 161, 7), // "version"
QT_MOC_LITERAL(13, 169, 11), // "downloadUrl"
QT_MOC_LITERAL(14, 181, 4), // "hash"
QT_MOC_LITERAL(15, 186, 11), // "firstSigner"
QT_MOC_LITERAL(16, 198, 12), // "secondSigner"
QT_MOC_LITERAL(17, 211, 12), // "miningStatus"
QT_MOC_LITERAL(18, 224, 8), // "isMining"
QT_MOC_LITERAL(19, 233, 19), // "proxyAddressChanged"
QT_MOC_LITERAL(20, 253, 12), // "createWallet"
QT_MOC_LITERAL(21, 266, 4), // "path"
QT_MOC_LITERAL(22, 271, 8), // "password"
QT_MOC_LITERAL(23, 280, 8), // "language"
QT_MOC_LITERAL(24, 289, 17), // "NetworkType::Type"
QT_MOC_LITERAL(25, 307, 7), // "nettype"
QT_MOC_LITERAL(26, 315, 9), // "kdfRounds"
QT_MOC_LITERAL(27, 325, 10), // "openWallet"
QT_MOC_LITERAL(28, 336, 15), // "openWalletAsync"
QT_MOC_LITERAL(29, 352, 14), // "recoveryWallet"
QT_MOC_LITERAL(30, 367, 4), // "seed"
QT_MOC_LITERAL(31, 372, 11), // "seed_offset"
QT_MOC_LITERAL(32, 384, 13), // "restoreHeight"
QT_MOC_LITERAL(33, 398, 20), // "createWalletFromKeys"
QT_MOC_LITERAL(34, 419, 7), // "address"
QT_MOC_LITERAL(35, 427, 7), // "viewkey"
QT_MOC_LITERAL(36, 435, 8), // "spendkey"
QT_MOC_LITERAL(37, 444, 22), // "createWalletFromDevice"
QT_MOC_LITERAL(38, 467, 10), // "deviceName"
QT_MOC_LITERAL(39, 478, 19), // "subaddressLookahead"
QT_MOC_LITERAL(40, 498, 27), // "createWalletFromDeviceAsync"
QT_MOC_LITERAL(41, 526, 11), // "closeWallet"
QT_MOC_LITERAL(42, 538, 16), // "closeWalletAsync"
QT_MOC_LITERAL(43, 555, 8), // "QJSValue"
QT_MOC_LITERAL(44, 564, 8), // "callback"
QT_MOC_LITERAL(45, 573, 12), // "walletExists"
QT_MOC_LITERAL(46, 586, 11), // "findWallets"
QT_MOC_LITERAL(47, 598, 11), // "errorString"
QT_MOC_LITERAL(48, 610, 13), // "displayAmount"
QT_MOC_LITERAL(49, 624, 6), // "amount"
QT_MOC_LITERAL(50, 631, 16), // "amountFromString"
QT_MOC_LITERAL(51, 648, 16), // "amountFromDouble"
QT_MOC_LITERAL(52, 665, 21), // "amountsSumFromStrings"
QT_MOC_LITERAL(53, 687, 16), // "QVector<QString>"
QT_MOC_LITERAL(54, 704, 7), // "amounts"
QT_MOC_LITERAL(55, 712, 20), // "maximumAllowedAmount"
QT_MOC_LITERAL(56, 733, 28), // "maximumAllowedAmountAsString"
QT_MOC_LITERAL(57, 762, 14), // "paymentIdValid"
QT_MOC_LITERAL(58, 777, 10), // "payment_id"
QT_MOC_LITERAL(59, 788, 12), // "addressValid"
QT_MOC_LITERAL(60, 801, 8), // "keyValid"
QT_MOC_LITERAL(61, 810, 3), // "key"
QT_MOC_LITERAL(62, 814, 9), // "isViewKey"
QT_MOC_LITERAL(63, 824, 20), // "paymentIdFromAddress"
QT_MOC_LITERAL(64, 845, 21), // "setDaemonAddressAsync"
QT_MOC_LITERAL(65, 867, 9), // "connected"
QT_MOC_LITERAL(66, 877, 17), // "networkDifficulty"
QT_MOC_LITERAL(67, 895, 16), // "blockchainHeight"
QT_MOC_LITERAL(68, 912, 22), // "blockchainTargetHeight"
QT_MOC_LITERAL(69, 935, 14), // "miningHashRate"
QT_MOC_LITERAL(70, 950, 17), // "localDaemonSynced"
QT_MOC_LITERAL(71, 968, 13), // "isDaemonLocal"
QT_MOC_LITERAL(72, 982, 14), // "daemon_address"
QT_MOC_LITERAL(73, 997, 17), // "miningStatusAsync"
QT_MOC_LITERAL(74, 1015, 11), // "startMining"
QT_MOC_LITERAL(75, 1027, 7), // "threads"
QT_MOC_LITERAL(76, 1035, 16), // "backgroundMining"
QT_MOC_LITERAL(77, 1052, 13), // "ignoreBattery"
QT_MOC_LITERAL(78, 1066, 10), // "stopMining"
QT_MOC_LITERAL(79, 1077, 14), // "urlToLocalPath"
QT_MOC_LITERAL(80, 1092, 3), // "url"
QT_MOC_LITERAL(81, 1096, 14), // "localPathToUrl"
QT_MOC_LITERAL(82, 1111, 11), // "setLogLevel"
QT_MOC_LITERAL(83, 1123, 8), // "logLevel"
QT_MOC_LITERAL(84, 1132, 16), // "setLogCategories"
QT_MOC_LITERAL(85, 1149, 10), // "categories"
QT_MOC_LITERAL(86, 1160, 3), // "add"
QT_MOC_LITERAL(87, 1164, 1), // "x"
QT_MOC_LITERAL(88, 1166, 1), // "y"
QT_MOC_LITERAL(89, 1168, 3), // "sub"
QT_MOC_LITERAL(90, 1172, 4), // "addi"
QT_MOC_LITERAL(91, 1177, 4), // "subi"
QT_MOC_LITERAL(92, 1182, 19), // "getPasswordStrength"
QT_MOC_LITERAL(93, 1202, 16), // "resolveOpenAlias"
QT_MOC_LITERAL(94, 1219, 9), // "parse_uri"
QT_MOC_LITERAL(95, 1229, 3), // "uri"
QT_MOC_LITERAL(96, 1233, 8), // "QString&"
QT_MOC_LITERAL(97, 1242, 9), // "uint64_t&"
QT_MOC_LITERAL(98, 1252, 14), // "tx_description"
QT_MOC_LITERAL(99, 1267, 14), // "recipient_name"
QT_MOC_LITERAL(100, 1282, 17), // "QVector<QString>&"
QT_MOC_LITERAL(101, 1300, 18), // "unknown_parameters"
QT_MOC_LITERAL(102, 1319, 5), // "error"
QT_MOC_LITERAL(103, 1325, 19), // "parse_uri_to_object"
QT_MOC_LITERAL(104, 1345, 8), // "make_uri"
QT_MOC_LITERAL(105, 1354, 10), // "saveQrCode"
QT_MOC_LITERAL(106, 1365, 21), // "saveQrCodeToClipboard"
QT_MOC_LITERAL(107, 1387, 17), // "checkUpdatesAsync"
QT_MOC_LITERAL(108, 1405, 8), // "software"
QT_MOC_LITERAL(109, 1414, 6), // "subdir"
QT_MOC_LITERAL(110, 1421, 8), // "buildTag"
QT_MOC_LITERAL(111, 1430, 12), // "checkUpdates"
QT_MOC_LITERAL(112, 1443, 16), // "clearWalletCache"
QT_MOC_LITERAL(113, 1460, 8), // "fileName"
QT_MOC_LITERAL(114, 1469, 19), // "onPassphraseEntered"
QT_MOC_LITERAL(115, 1489, 10), // "passphrase"
QT_MOC_LITERAL(116, 1500, 15), // "enter_on_device"
QT_MOC_LITERAL(117, 1516, 11), // "entry_abort"
QT_MOC_LITERAL(118, 1528, 12) // "proxyAddress"

    },
    "WalletManager\0walletOpened\0\0Wallet*\0"
    "wallet\0walletCreated\0walletPassphraseNeeded\0"
    "onDevice\0deviceButtonRequest\0buttonCode\0"
    "deviceButtonPressed\0checkUpdatesComplete\0"
    "version\0downloadUrl\0hash\0firstSigner\0"
    "secondSigner\0miningStatus\0isMining\0"
    "proxyAddressChanged\0createWallet\0path\0"
    "password\0language\0NetworkType::Type\0"
    "nettype\0kdfRounds\0openWallet\0"
    "openWalletAsync\0recoveryWallet\0seed\0"
    "seed_offset\0restoreHeight\0"
    "createWalletFromKeys\0address\0viewkey\0"
    "spendkey\0createWalletFromDevice\0"
    "deviceName\0subaddressLookahead\0"
    "createWalletFromDeviceAsync\0closeWallet\0"
    "closeWalletAsync\0QJSValue\0callback\0"
    "walletExists\0findWallets\0errorString\0"
    "displayAmount\0amount\0amountFromString\0"
    "amountFromDouble\0amountsSumFromStrings\0"
    "QVector<QString>\0amounts\0maximumAllowedAmount\0"
    "maximumAllowedAmountAsString\0"
    "paymentIdValid\0payment_id\0addressValid\0"
    "keyValid\0key\0isViewKey\0paymentIdFromAddress\0"
    "setDaemonAddressAsync\0connected\0"
    "networkDifficulty\0blockchainHeight\0"
    "blockchainTargetHeight\0miningHashRate\0"
    "localDaemonSynced\0isDaemonLocal\0"
    "daemon_address\0miningStatusAsync\0"
    "startMining\0threads\0backgroundMining\0"
    "ignoreBattery\0stopMining\0urlToLocalPath\0"
    "url\0localPathToUrl\0setLogLevel\0logLevel\0"
    "setLogCategories\0categories\0add\0x\0y\0"
    "sub\0addi\0subi\0getPasswordStrength\0"
    "resolveOpenAlias\0parse_uri\0uri\0QString&\0"
    "uint64_t&\0tx_description\0recipient_name\0"
    "QVector<QString>&\0unknown_parameters\0"
    "error\0parse_uri_to_object\0make_uri\0"
    "saveQrCode\0saveQrCodeToClipboard\0"
    "checkUpdatesAsync\0software\0subdir\0"
    "buildTag\0checkUpdates\0clearWalletCache\0"
    "fileName\0onPassphraseEntered\0passphrase\0"
    "enter_on_device\0entry_abort\0proxyAddress"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_WalletManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      82,   14, // methods
       2,  902, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  424,    2, 0x06 /* Public */,
       5,    1,  427,    2, 0x06 /* Public */,
       6,    1,  430,    2, 0x06 /* Public */,
       8,    1,  433,    2, 0x06 /* Public */,
      10,    0,  436,    2, 0x06 /* Public */,
      11,    5,  437,    2, 0x06 /* Public */,
      17,    1,  448,    2, 0x06 /* Public */,
      19,    0,  451,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      20,    5,  452,    2, 0x02 /* Public */,
      20,    4,  463,    2, 0x22 /* Public | MethodCloned */,
      20,    3,  472,    2, 0x22 /* Public | MethodCloned */,
      27,    4,  479,    2, 0x02 /* Public */,
      27,    3,  488,    2, 0x22 /* Public | MethodCloned */,
      27,    2,  495,    2, 0x22 /* Public | MethodCloned */,
      28,    4,  500,    2, 0x02 /* Public */,
      28,    3,  509,    2, 0x22 /* Public | MethodCloned */,
      28,    2,  516,    2, 0x22 /* Public | MethodCloned */,
      29,    6,  521,    2, 0x02 /* Public */,
      29,    5,  534,    2, 0x22 /* Public | MethodCloned */,
      29,    4,  545,    2, 0x22 /* Public | MethodCloned */,
      29,    3,  554,    2, 0x22 /* Public | MethodCloned */,
      33,    8,  561,    2, 0x02 /* Public */,
      33,    7,  578,    2, 0x22 /* Public | MethodCloned */,
      33,    6,  593,    2, 0x22 /* Public | MethodCloned */,
      33,    5,  606,    2, 0x22 /* Public | MethodCloned */,
      37,    7,  617,    2, 0x02 /* Public */,
      37,    6,  632,    2, 0x22 /* Public | MethodCloned */,
      37,    5,  645,    2, 0x22 /* Public | MethodCloned */,
      37,    4,  656,    2, 0x22 /* Public | MethodCloned */,
      40,    7,  665,    2, 0x02 /* Public */,
      40,    6,  680,    2, 0x22 /* Public | MethodCloned */,
      40,    5,  693,    2, 0x22 /* Public | MethodCloned */,
      40,    4,  704,    2, 0x22 /* Public | MethodCloned */,
      41,    0,  713,    2, 0x02 /* Public */,
      42,    1,  714,    2, 0x02 /* Public */,
      45,    1,  717,    2, 0x02 /* Public */,
      46,    1,  720,    2, 0x02 /* Public */,
      47,    0,  723,    2, 0x02 /* Public */,
      48,    1,  724,    2, 0x02 /* Public */,
      50,    1,  727,    2, 0x02 /* Public */,
      51,    1,  730,    2, 0x02 /* Public */,
      52,    1,  733,    2, 0x02 /* Public */,
      55,    0,  736,    2, 0x02 /* Public */,
      56,    0,  737,    2, 0x02 /* Public */,
      57,    1,  738,    2, 0x02 /* Public */,
      59,    2,  741,    2, 0x02 /* Public */,
      60,    4,  746,    2, 0x02 /* Public */,
      63,    2,  755,    2, 0x02 /* Public */,
      64,    1,  760,    2, 0x02 /* Public */,
      65,    0,  763,    2, 0x02 /* Public */,
      66,    0,  764,    2, 0x02 /* Public */,
      67,    0,  765,    2, 0x02 /* Public */,
      68,    0,  766,    2, 0x02 /* Public */,
      69,    0,  767,    2, 0x02 /* Public */,
      70,    0,  768,    2, 0x02 /* Public */,
      71,    1,  769,    2, 0x02 /* Public */,
      73,    0,  772,    2, 0x02 /* Public */,
      74,    4,  773,    2, 0x02 /* Public */,
      78,    0,  782,    2, 0x02 /* Public */,
      79,    1,  783,    2, 0x02 /* Public */,
      81,    1,  786,    2, 0x02 /* Public */,
      82,    1,  789,    2, 0x02 /* Public */,
      84,    1,  792,    2, 0x02 /* Public */,
      86,    2,  795,    2, 0x02 /* Public */,
      89,    2,  800,    2, 0x02 /* Public */,
      90,    2,  805,    2, 0x02 /* Public */,
      91,    2,  810,    2, 0x02 /* Public */,
      92,    1,  815,    2, 0x02 /* Public */,
      93,    1,  818,    2, 0x02 /* Public */,
      94,    8,  821,    2, 0x02 /* Public */,
     103,    1,  838,    2, 0x02 /* Public */,
     104,    4,  841,    2, 0x02 /* Public */,
     104,    3,  850,    2, 0x22 /* Public | MethodCloned */,
     104,    2,  857,    2, 0x22 /* Public | MethodCloned */,
     104,    1,  862,    2, 0x22 /* Public | MethodCloned */,
     105,    2,  865,    2, 0x02 /* Public */,
     106,    1,  870,    2, 0x02 /* Public */,
     107,    4,  873,    2, 0x02 /* Public */,
     111,    2,  882,    2, 0x02 /* Public */,
     112,    1,  887,    2, 0x02 /* Public */,
     114,    3,  890,    2, 0x02 /* Public */,
     114,    2,  897,    2, 0x22 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::ULongLong,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   12,   13,   14,   15,   16,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,

 // methods: parameters
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::ULongLong,   21,   22,   23,   25,   26,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 24,   21,   22,   23,   25,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString,   21,   22,   23,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::ULongLong,   21,   22,   25,   26,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24,   21,   22,   25,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString,   21,   22,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::ULongLong,   21,   22,   25,   26,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 24,   21,   22,   25,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   21,   22,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::ULongLong, QMetaType::ULongLong,   21,   30,   31,   25,   32,   26,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::ULongLong,   21,   30,   31,   25,   32,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 24,   21,   30,   31,   25,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, QMetaType::QString,   21,   30,   31,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::ULongLong, QMetaType::ULongLong,   21,   23,   25,   34,   35,   36,   32,   26,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::ULongLong,   21,   23,   25,   34,   35,   36,   32,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::QString, QMetaType::QString,   21,   23,   25,   34,   35,   36,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::QString,   21,   23,   25,   34,   35,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString, QMetaType::ULongLong,   21,   22,   25,   38,   32,   39,   26,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString,   21,   22,   25,   38,   32,   39,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::ULongLong,   21,   22,   25,   38,   32,
    0x80000000 | 3, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString,   21,   22,   25,   38,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString, QMetaType::ULongLong,   21,   22,   25,   38,   32,   39,   26,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString,   21,   22,   25,   38,   32,   39,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString, QMetaType::ULongLong,   21,   22,   25,   38,   32,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 24, QMetaType::QString,   21,   22,   25,   38,
    QMetaType::QString,
    QMetaType::Void, 0x80000000 | 43,   44,
    QMetaType::Bool, QMetaType::QString,   21,
    QMetaType::QStringList, QMetaType::QString,   21,
    QMetaType::QString,
    QMetaType::QString, QMetaType::ULongLong,   49,
    QMetaType::ULongLong, QMetaType::QString,   49,
    QMetaType::ULongLong, QMetaType::Double,   49,
    QMetaType::QString, 0x80000000 | 53,   54,
    QMetaType::ULongLong,
    QMetaType::QString,
    QMetaType::Bool, QMetaType::QString,   58,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 24,   34,   25,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::Bool, 0x80000000 | 24,   61,   34,   62,   25,
    QMetaType::QString, QMetaType::QString, 0x80000000 | 24,   34,   25,
    QMetaType::Void, QMetaType::QString,   34,
    QMetaType::Bool,
    QMetaType::ULongLong,
    QMetaType::ULongLong,
    QMetaType::ULongLong,
    QMetaType::Double,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::QString,   72,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::UInt, QMetaType::Bool, QMetaType::Bool,   34,   75,   76,   77,
    QMetaType::Bool,
    QMetaType::QString, QMetaType::QUrl,   80,
    QMetaType::QUrl, QMetaType::QString,   21,
    QMetaType::Void, QMetaType::Int,   83,
    QMetaType::Void, QMetaType::QString,   85,
    QMetaType::ULongLong, QMetaType::ULongLong, QMetaType::ULongLong,   87,   88,
    QMetaType::ULongLong, QMetaType::ULongLong, QMetaType::ULongLong,   87,   88,
    QMetaType::LongLong, QMetaType::LongLong, QMetaType::LongLong,   87,   88,
    QMetaType::LongLong, QMetaType::LongLong, QMetaType::LongLong,   87,   88,
    QMetaType::Double, QMetaType::QString,   22,
    QMetaType::QString, QMetaType::QString,   34,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 96, 0x80000000 | 96, 0x80000000 | 97, 0x80000000 | 96, 0x80000000 | 96, 0x80000000 | 100, 0x80000000 | 96,   95,   34,   58,   49,   98,   99,  101,  102,
    QMetaType::QVariantMap, QMetaType::QString,   95,
    QMetaType::QString, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString, QMetaType::QString,   34,   49,   98,   99,
    QMetaType::QString, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString,   34,   49,   98,
    QMetaType::QString, QMetaType::QString, QMetaType::ULongLong,   34,   49,
    QMetaType::QString, QMetaType::QString,   34,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,  108,  109,  110,   12,
    QMetaType::QString, QMetaType::QString, QMetaType::QString,  108,  109,
    QMetaType::Bool, QMetaType::QString,  113,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,  115,  116,  117,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,  115,  116,

 // properties: name, type, flags
      65, QMetaType::Bool, 0x00095001,
     118, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       7,

       0        // eod
};

void WalletManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<WalletManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->walletOpened((*reinterpret_cast< Wallet*(*)>(_a[1]))); break;
        case 1: _t->walletCreated((*reinterpret_cast< Wallet*(*)>(_a[1]))); break;
        case 2: _t->walletPassphraseNeeded((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->deviceButtonRequest((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 4: _t->deviceButtonPressed(); break;
        case 5: _t->checkUpdatesComplete((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5]))); break;
        case 6: _t->miningStatus((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->proxyAddressChanged(); break;
        case 8: { Wallet* _r = _t->createWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< NetworkType::Type(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 9: { Wallet* _r = _t->createWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< NetworkType::Type(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 10: { Wallet* _r = _t->createWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 11: { Wallet* _r = _t->openWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< quint64(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 12: { Wallet* _r = _t->openWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 13: { Wallet* _r = _t->openWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 14: _t->openWalletAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< quint64(*)>(_a[4]))); break;
        case 15: _t->openWalletAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3]))); break;
        case 16: _t->openWalletAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 17: { Wallet* _r = _t->recoveryWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< NetworkType::Type(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])),(*reinterpret_cast< quint64(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 18: { Wallet* _r = _t->recoveryWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< NetworkType::Type(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 19: { Wallet* _r = _t->recoveryWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< NetworkType::Type(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 20: { Wallet* _r = _t->recoveryWallet((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 21: { Wallet* _r = _t->createWalletFromKeys((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])),(*reinterpret_cast< quint64(*)>(_a[7])),(*reinterpret_cast< quint64(*)>(_a[8])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 22: { Wallet* _r = _t->createWalletFromKeys((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])),(*reinterpret_cast< quint64(*)>(_a[7])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 23: { Wallet* _r = _t->createWalletFromKeys((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 24: { Wallet* _r = _t->createWalletFromKeys((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 25: { Wallet* _r = _t->createWalletFromDevice((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])),(*reinterpret_cast< quint64(*)>(_a[7])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 26: { Wallet* _r = _t->createWalletFromDevice((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 27: { Wallet* _r = _t->createWalletFromDevice((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 28: { Wallet* _r = _t->createWalletFromDevice((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< Wallet**>(_a[0]) = std::move(_r); }  break;
        case 29: _t->createWalletFromDeviceAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6])),(*reinterpret_cast< quint64(*)>(_a[7]))); break;
        case 30: _t->createWalletFromDeviceAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5])),(*reinterpret_cast< const QString(*)>(_a[6]))); break;
        case 31: _t->createWalletFromDeviceAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< quint64(*)>(_a[5]))); break;
        case 32: _t->createWalletFromDeviceAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< NetworkType::Type(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 33: { QString _r = _t->closeWallet();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->closeWalletAsync((*reinterpret_cast< const QJSValue(*)>(_a[1]))); break;
        case 35: { bool _r = _t->walletExists((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { QStringList _r = _t->findWallets((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 37: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 38: { QString _r = _t->displayAmount((*reinterpret_cast< quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 39: { quint64 _r = _t->amountFromString((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 40: { quint64 _r = _t->amountFromDouble((*reinterpret_cast< double(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 41: { QString _r = _t->amountsSumFromStrings((*reinterpret_cast< const QVector<QString>(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 42: { quint64 _r = _t->maximumAllowedAmount();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 43: { QString _r = _t->maximumAllowedAmountAsString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 44: { bool _r = _t->paymentIdValid((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 45: { bool _r = _t->addressValid((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 46: { bool _r = _t->keyValid((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< NetworkType::Type(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 47: { QString _r = _t->paymentIdFromAddress((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->setDaemonAddressAsync((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 49: { bool _r = _t->connected();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 50: { quint64 _r = _t->networkDifficulty();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 51: { quint64 _r = _t->blockchainHeight();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 52: { quint64 _r = _t->blockchainTargetHeight();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 53: { double _r = _t->miningHashRate();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 54: { bool _r = _t->localDaemonSynced();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 55: { bool _r = _t->isDaemonLocal((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 56: _t->miningStatusAsync(); break;
        case 57: { bool _r = _t->startMining((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 58: { bool _r = _t->stopMining();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 59: { QString _r = _t->urlToLocalPath((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 60: { QUrl _r = _t->localPathToUrl((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = std::move(_r); }  break;
        case 61: _t->setLogLevel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 62: _t->setLogCategories((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 63: { quint64 _r = _t->add((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 64: { quint64 _r = _t->sub((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 65: { qint64 _r = _t->addi((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< qint64*>(_a[0]) = std::move(_r); }  break;
        case 66: { qint64 _r = _t->subi((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< qint64*>(_a[0]) = std::move(_r); }  break;
        case 67: { double _r = _t->getPasswordStrength((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 68: { QString _r = _t->resolveOpenAlias((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 69: { bool _r = _t->parse_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3])),(*reinterpret_cast< uint64_t(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6])),(*reinterpret_cast< QVector<QString>(*)>(_a[7])),(*reinterpret_cast< QString(*)>(_a[8])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 70: { QVariantMap _r = _t->parse_uri_to_object((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 71: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const quint64(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 72: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const quint64(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 73: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const quint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 74: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 75: { bool _r = _t->saveQrCode((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 76: _t->saveQrCodeToClipboard((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 77: _t->checkUpdatesAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 78: { QString _r = _t->checkUpdates((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 79: { bool _r = _t->clearWalletCache((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 80: _t->onPassphraseEntered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 81: _t->onPassphraseEntered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<QString> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (WalletManager::*)(Wallet * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::walletOpened)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)(Wallet * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::walletCreated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::walletPassphraseNeeded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)(quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::deviceButtonRequest)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::deviceButtonPressed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)(const QString & , const QString & , const QString & , const QString & , const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::checkUpdatesComplete)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)(bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::miningStatus)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (WalletManager::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&WalletManager::proxyAddressChanged)) {
                *result = 7;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<WalletManager *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->connected(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->proxyAddress(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<WalletManager *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setProxyAddress(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject WalletManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_WalletManager.data,
    qt_meta_data_WalletManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *WalletManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WalletManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_WalletManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PassprasePrompter"))
        return static_cast< PassprasePrompter*>(this);
    return QObject::qt_metacast(_clname);
}

int WalletManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 82)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 82;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 82)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 82;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void WalletManager::walletOpened(Wallet * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void WalletManager::walletCreated(Wallet * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void WalletManager::walletPassphraseNeeded(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void WalletManager::deviceButtonRequest(quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void WalletManager::deviceButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void WalletManager::checkUpdatesComplete(const QString & _t1, const QString & _t2, const QString & _t3, const QString & _t4, const QString & _t5)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(const_cast< WalletManager *>(this), &staticMetaObject, 5, _a);
}

// SIGNAL 6
void WalletManager::miningStatus(bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< WalletManager *>(this), &staticMetaObject, 6, _a);
}

// SIGNAL 7
void WalletManager::proxyAddressChanged()const
{
    QMetaObject::activate(const_cast< WalletManager *>(this), &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
