/****************************************************************************
** Meta object code from reading C++ file 'DaemonManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/daemon/DaemonManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DaemonManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DaemonManager_t {
    QByteArrayData data[33];
    char stringdata0[393];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DaemonManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DaemonManager_t qt_meta_stringdata_DaemonManager = {
    {
QT_MOC_LITERAL(0, 0, 13), // "DaemonManager"
QT_MOC_LITERAL(1, 14, 13), // "daemonStarted"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 13), // "daemonStopped"
QT_MOC_LITERAL(4, 43, 18), // "daemonStartFailure"
QT_MOC_LITERAL(5, 62, 5), // "error"
QT_MOC_LITERAL(6, 68, 20), // "daemonConsoleUpdated"
QT_MOC_LITERAL(7, 89, 7), // "message"
QT_MOC_LITERAL(8, 97, 11), // "printOutput"
QT_MOC_LITERAL(9, 109, 10), // "printError"
QT_MOC_LITERAL(10, 120, 12), // "stateChanged"
QT_MOC_LITERAL(11, 133, 22), // "QProcess::ProcessState"
QT_MOC_LITERAL(12, 156, 5), // "state"
QT_MOC_LITERAL(13, 162, 5), // "start"
QT_MOC_LITERAL(14, 168, 5), // "flags"
QT_MOC_LITERAL(15, 174, 17), // "NetworkType::Type"
QT_MOC_LITERAL(16, 192, 7), // "nettype"
QT_MOC_LITERAL(17, 200, 7), // "dataDir"
QT_MOC_LITERAL(18, 208, 20), // "bootstrapNodeAddress"
QT_MOC_LITERAL(19, 229, 6), // "noSync"
QT_MOC_LITERAL(20, 236, 15), // "pruneBlockchain"
QT_MOC_LITERAL(21, 252, 9), // "stopAsync"
QT_MOC_LITERAL(22, 262, 8), // "QJSValue"
QT_MOC_LITERAL(23, 271, 8), // "callback"
QT_MOC_LITERAL(24, 280, 12), // "runningAsync"
QT_MOC_LITERAL(25, 293, 16), // "sendCommandAsync"
QT_MOC_LITERAL(26, 310, 3), // "cmd"
QT_MOC_LITERAL(27, 314, 4), // "exit"
QT_MOC_LITERAL(28, 319, 15), // "validateDataDir"
QT_MOC_LITERAL(29, 335, 25), // "estimatedBlockchainSizeGb"
QT_MOC_LITERAL(30, 361, 15), // "checkLmdbExists"
QT_MOC_LITERAL(31, 377, 7), // "datadir"
QT_MOC_LITERAL(32, 385, 7) // "getArgs"

    },
    "DaemonManager\0daemonStarted\0\0daemonStopped\0"
    "daemonStartFailure\0error\0daemonConsoleUpdated\0"
    "message\0printOutput\0printError\0"
    "stateChanged\0QProcess::ProcessState\0"
    "state\0start\0flags\0NetworkType::Type\0"
    "nettype\0dataDir\0bootstrapNodeAddress\0"
    "noSync\0pruneBlockchain\0stopAsync\0"
    "QJSValue\0callback\0runningAsync\0"
    "sendCommandAsync\0cmd\0exit\0validateDataDir\0"
    "estimatedBlockchainSizeGb\0checkLmdbExists\0"
    "datadir\0getArgs"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DaemonManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  114,    2, 0x06 /* Public */,
       3,    0,  115,    2, 0x06 /* Public */,
       4,    1,  116,    2, 0x06 /* Public */,
       6,    1,  119,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,  122,    2, 0x0a /* Public */,
       9,    0,  123,    2, 0x0a /* Public */,
      10,    1,  124,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      13,    6,  127,    2, 0x02 /* Public */,
      13,    5,  140,    2, 0x22 /* Public | MethodCloned */,
      13,    4,  151,    2, 0x22 /* Public | MethodCloned */,
      13,    3,  160,    2, 0x22 /* Public | MethodCloned */,
      13,    2,  167,    2, 0x22 /* Public | MethodCloned */,
      21,    3,  172,    2, 0x02 /* Public */,
      19,    0,  179,    2, 0x02 /* Public */,
      24,    3,  180,    2, 0x02 /* Public */,
      25,    4,  187,    2, 0x02 /* Public */,
      27,    0,  196,    2, 0x02 /* Public */,
      28,    2,  197,    2, 0x02 /* Public */,
      30,    1,  202,    2, 0x02 /* Public */,
      32,    1,  205,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    7,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,

 // methods: parameters
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 15, QMetaType::QString, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,   14,   16,   17,   18,   19,   20,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 15, QMetaType::QString, QMetaType::QString, QMetaType::Bool,   14,   16,   17,   18,   19,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 15, QMetaType::QString, QMetaType::QString,   14,   16,   17,   18,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 15, QMetaType::QString,   14,   16,   17,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 15,   14,   16,
    QMetaType::Void, 0x80000000 | 15, QMetaType::QString, 0x80000000 | 22,   16,   17,   23,
    QMetaType::Bool,
    QMetaType::Void, 0x80000000 | 15, QMetaType::QString, 0x80000000 | 22,   16,   17,   23,
    QMetaType::Void, QMetaType::QStringList, 0x80000000 | 15, QMetaType::QString, 0x80000000 | 22,   26,   16,   17,   23,
    QMetaType::Void,
    QMetaType::QVariantMap, QMetaType::QString, QMetaType::Int,   17,   29,
    QMetaType::Bool, QMetaType::QString,   31,
    QMetaType::QString, QMetaType::QString,   17,

       0        // eod
};

void DaemonManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DaemonManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->daemonStarted(); break;
        case 1: _t->daemonStopped(); break;
        case 2: _t->daemonStartFailure((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->daemonConsoleUpdated((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->printOutput(); break;
        case 5: _t->printError(); break;
        case 6: _t->stateChanged((*reinterpret_cast< QProcess::ProcessState(*)>(_a[1]))); break;
        case 7: { bool _r = _t->start((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< bool(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->start((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->start((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->start((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->start((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->stopAsync((*reinterpret_cast< NetworkType::Type(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QJSValue(*)>(_a[3]))); break;
        case 13: { bool _r = _t->noSync();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->runningAsync((*reinterpret_cast< NetworkType::Type(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QJSValue(*)>(_a[3]))); break;
        case 15: _t->sendCommandAsync((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< NetworkType::Type(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QJSValue(*)>(_a[4]))); break;
        case 16: _t->exit(); break;
        case 17: { QVariantMap _r = _t->validateDataDir((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->checkLmdbExists((*reinterpret_cast< QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->getArgs((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DaemonManager::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DaemonManager::daemonStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DaemonManager::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DaemonManager::daemonStopped)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DaemonManager::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DaemonManager::daemonStartFailure)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DaemonManager::*)(QString ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DaemonManager::daemonConsoleUpdated)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DaemonManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DaemonManager.data,
    qt_meta_data_DaemonManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DaemonManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DaemonManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DaemonManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DaemonManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void DaemonManager::daemonStarted()const
{
    QMetaObject::activate(const_cast< DaemonManager *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DaemonManager::daemonStopped()const
{
    QMetaObject::activate(const_cast< DaemonManager *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DaemonManager::daemonStartFailure(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DaemonManager *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DaemonManager::daemonConsoleUpdated(QString _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DaemonManager *>(this), &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
