/****************************************************************************
** Meta object code from reading C++ file 'Wallet.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/libwalletqt/Wallet.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Wallet.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Wallet_t {
    QByteArrayData data[229];
    char stringdata0[3429];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Wallet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Wallet_t qt_meta_stringdata_Wallet = {
    {
QT_MOC_LITERAL(0, 0, 6), // "Wallet"
QT_MOC_LITERAL(1, 7, 7), // "updated"
QT_MOC_LITERAL(2, 15, 0), // ""
QT_MOC_LITERAL(3, 16, 9), // "refreshed"
QT_MOC_LITERAL(4, 26, 10), // "moneySpent"
QT_MOC_LITERAL(5, 37, 4), // "txId"
QT_MOC_LITERAL(6, 42, 6), // "amount"
QT_MOC_LITERAL(7, 49, 13), // "moneyReceived"
QT_MOC_LITERAL(8, 63, 24), // "unconfirmedMoneyReceived"
QT_MOC_LITERAL(9, 88, 8), // "newBlock"
QT_MOC_LITERAL(10, 97, 6), // "height"
QT_MOC_LITERAL(11, 104, 12), // "targetHeight"
QT_MOC_LITERAL(12, 117, 19), // "backgroundSyncSetup"
QT_MOC_LITERAL(13, 137, 21), // "backgroundSyncStarted"
QT_MOC_LITERAL(14, 159, 21), // "backgroundSyncStopped"
QT_MOC_LITERAL(15, 181, 18), // "addressBookChanged"
QT_MOC_LITERAL(16, 200, 19), // "historyModelChanged"
QT_MOC_LITERAL(17, 220, 27), // "walletCreationHeightChanged"
QT_MOC_LITERAL(18, 248, 19), // "deviceButtonRequest"
QT_MOC_LITERAL(19, 268, 10), // "buttonCode"
QT_MOC_LITERAL(20, 279, 19), // "deviceButtonPressed"
QT_MOC_LITERAL(21, 299, 22), // "walletPassphraseNeeded"
QT_MOC_LITERAL(22, 322, 8), // "onDevice"
QT_MOC_LITERAL(23, 331, 20), // "transactionCommitted"
QT_MOC_LITERAL(24, 352, 6), // "status"
QT_MOC_LITERAL(25, 359, 19), // "PendingTransaction*"
QT_MOC_LITERAL(26, 379, 1), // "t"
QT_MOC_LITERAL(27, 381, 4), // "txid"
QT_MOC_LITERAL(28, 386, 15), // "heightRefreshed"
QT_MOC_LITERAL(29, 402, 12), // "walletHeight"
QT_MOC_LITERAL(30, 415, 12), // "daemonHeight"
QT_MOC_LITERAL(31, 428, 23), // "deviceShowAddressShowed"
QT_MOC_LITERAL(32, 452, 18), // "transactionCreated"
QT_MOC_LITERAL(33, 471, 11), // "transaction"
QT_MOC_LITERAL(34, 483, 16), // "QVector<QString>"
QT_MOC_LITERAL(35, 500, 9), // "addresses"
QT_MOC_LITERAL(36, 510, 9), // "paymentId"
QT_MOC_LITERAL(37, 520, 10), // "mixinCount"
QT_MOC_LITERAL(38, 531, 23), // "connectionStatusChanged"
QT_MOC_LITERAL(39, 555, 31), // "currentSubaddressAccountChanged"
QT_MOC_LITERAL(40, 587, 19), // "disconnectedChanged"
QT_MOC_LITERAL(41, 607, 19), // "proxyAddressChanged"
QT_MOC_LITERAL(42, 627, 17), // "refreshingChanged"
QT_MOC_LITERAL(43, 645, 15), // "setSeedLanguage"
QT_MOC_LITERAL(44, 661, 4), // "lang"
QT_MOC_LITERAL(45, 666, 9), // "connected"
QT_MOC_LITERAL(46, 676, 16), // "ConnectionStatus"
QT_MOC_LITERAL(47, 693, 10), // "forceCheck"
QT_MOC_LITERAL(48, 704, 11), // "setPassword"
QT_MOC_LITERAL(49, 716, 8), // "password"
QT_MOC_LITERAL(50, 725, 7), // "address"
QT_MOC_LITERAL(51, 733, 12), // "accountIndex"
QT_MOC_LITERAL(52, 746, 12), // "addressIndex"
QT_MOC_LITERAL(53, 759, 10), // "storeAsync"
QT_MOC_LITERAL(54, 770, 8), // "QJSValue"
QT_MOC_LITERAL(55, 779, 8), // "callback"
QT_MOC_LITERAL(56, 788, 4), // "path"
QT_MOC_LITERAL(57, 793, 9), // "initAsync"
QT_MOC_LITERAL(58, 803, 13), // "daemonAddress"
QT_MOC_LITERAL(59, 817, 13), // "trustedDaemon"
QT_MOC_LITERAL(60, 831, 21), // "upperTransactionLimit"
QT_MOC_LITERAL(61, 853, 12), // "isRecovering"
QT_MOC_LITERAL(62, 866, 22), // "isRecoveringFromDevice"
QT_MOC_LITERAL(63, 889, 13), // "restoreHeight"
QT_MOC_LITERAL(64, 903, 12), // "proxyAddress"
QT_MOC_LITERAL(65, 916, 14), // "setDaemonLogin"
QT_MOC_LITERAL(66, 931, 14), // "daemonUsername"
QT_MOC_LITERAL(67, 946, 14), // "daemonPassword"
QT_MOC_LITERAL(68, 961, 14), // "createViewOnly"
QT_MOC_LITERAL(69, 976, 15), // "connectToDaemon"
QT_MOC_LITERAL(70, 992, 16), // "setTrustedDaemon"
QT_MOC_LITERAL(71, 1009, 3), // "arg"
QT_MOC_LITERAL(72, 1013, 7), // "balance"
QT_MOC_LITERAL(73, 1021, 10), // "balanceAll"
QT_MOC_LITERAL(74, 1032, 15), // "unlockedBalance"
QT_MOC_LITERAL(75, 1048, 18), // "unlockedBalanceAll"
QT_MOC_LITERAL(76, 1067, 23), // "switchSubaddressAccount"
QT_MOC_LITERAL(77, 1091, 20), // "addSubaddressAccount"
QT_MOC_LITERAL(78, 1112, 5), // "label"
QT_MOC_LITERAL(79, 1118, 21), // "numSubaddressAccounts"
QT_MOC_LITERAL(80, 1140, 15), // "numSubaddresses"
QT_MOC_LITERAL(81, 1156, 13), // "addSubaddress"
QT_MOC_LITERAL(82, 1170, 18), // "getSubaddressLabel"
QT_MOC_LITERAL(83, 1189, 18), // "setSubaddressLabel"
QT_MOC_LITERAL(84, 1208, 22), // "deviceShowAddressAsync"
QT_MOC_LITERAL(85, 1231, 10), // "isHwBacked"
QT_MOC_LITERAL(86, 1242, 8), // "isLedger"
QT_MOC_LITERAL(87, 1251, 8), // "isTrezor"
QT_MOC_LITERAL(88, 1260, 8), // "viewOnly"
QT_MOC_LITERAL(89, 1269, 18), // "refreshHeightAsync"
QT_MOC_LITERAL(90, 1288, 15), // "exportKeyImages"
QT_MOC_LITERAL(91, 1304, 3), // "all"
QT_MOC_LITERAL(92, 1308, 15), // "importKeyImages"
QT_MOC_LITERAL(93, 1324, 13), // "exportOutputs"
QT_MOC_LITERAL(94, 1338, 13), // "importOutputs"
QT_MOC_LITERAL(95, 1352, 16), // "scanTransactions"
QT_MOC_LITERAL(96, 1369, 5), // "txids"
QT_MOC_LITERAL(97, 1375, 19), // "setupBackgroundSync"
QT_MOC_LITERAL(98, 1395, 18), // "BackgroundSyncType"
QT_MOC_LITERAL(99, 1414, 20), // "background_sync_type"
QT_MOC_LITERAL(100, 1435, 15), // "wallet_password"
QT_MOC_LITERAL(101, 1451, 21), // "getBackgroundSyncType"
QT_MOC_LITERAL(102, 1473, 18), // "isBackgroundWallet"
QT_MOC_LITERAL(103, 1492, 19), // "isBackgroundSyncing"
QT_MOC_LITERAL(104, 1512, 19), // "startBackgroundSync"
QT_MOC_LITERAL(105, 1532, 18), // "stopBackgroundSync"
QT_MOC_LITERAL(106, 1551, 7), // "refresh"
QT_MOC_LITERAL(107, 1559, 22), // "historyAndSubaddresses"
QT_MOC_LITERAL(108, 1582, 12), // "startRefresh"
QT_MOC_LITERAL(109, 1595, 12), // "pauseRefresh"
QT_MOC_LITERAL(110, 1608, 22), // "createTransactionAsync"
QT_MOC_LITERAL(111, 1631, 20), // "destinationAddresses"
QT_MOC_LITERAL(112, 1652, 10), // "payment_id"
QT_MOC_LITERAL(113, 1663, 18), // "destinationAmounts"
QT_MOC_LITERAL(114, 1682, 11), // "mixin_count"
QT_MOC_LITERAL(115, 1694, 28), // "PendingTransaction::Priority"
QT_MOC_LITERAL(116, 1723, 8), // "priority"
QT_MOC_LITERAL(117, 1732, 20), // "createTransactionAll"
QT_MOC_LITERAL(118, 1753, 8), // "dst_addr"
QT_MOC_LITERAL(119, 1762, 25), // "createTransactionAllAsync"
QT_MOC_LITERAL(120, 1788, 31), // "createSweepUnmixableTransaction"
QT_MOC_LITERAL(121, 1820, 36), // "createSweepUnmixableTransacti..."
QT_MOC_LITERAL(122, 1857, 10), // "loadTxFile"
QT_MOC_LITERAL(123, 1868, 20), // "UnsignedTransaction*"
QT_MOC_LITERAL(124, 1889, 8), // "fileName"
QT_MOC_LITERAL(125, 1898, 12), // "submitTxFile"
QT_MOC_LITERAL(126, 1911, 22), // "commitTransactionAsync"
QT_MOC_LITERAL(127, 1934, 18), // "disposeTransaction"
QT_MOC_LITERAL(128, 1953, 27), // "estimateTransactionFeeAsync"
QT_MOC_LITERAL(129, 1981, 16), // "QVector<quint64>"
QT_MOC_LITERAL(130, 1998, 7), // "amounts"
QT_MOC_LITERAL(131, 2006, 17), // "generatePaymentId"
QT_MOC_LITERAL(132, 2024, 17), // "integratedAddress"
QT_MOC_LITERAL(133, 2042, 11), // "signMessage"
QT_MOC_LITERAL(134, 2054, 7), // "message"
QT_MOC_LITERAL(135, 2062, 8), // "filename"
QT_MOC_LITERAL(136, 2071, 19), // "verifySignedMessage"
QT_MOC_LITERAL(137, 2091, 9), // "signature"
QT_MOC_LITERAL(138, 2101, 9), // "parse_uri"
QT_MOC_LITERAL(139, 2111, 3), // "uri"
QT_MOC_LITERAL(140, 2115, 8), // "QString&"
QT_MOC_LITERAL(141, 2124, 9), // "uint64_t&"
QT_MOC_LITERAL(142, 2134, 14), // "tx_description"
QT_MOC_LITERAL(143, 2149, 14), // "recipient_name"
QT_MOC_LITERAL(144, 2164, 17), // "QVector<QString>&"
QT_MOC_LITERAL(145, 2182, 18), // "unknown_parameters"
QT_MOC_LITERAL(146, 2201, 5), // "error"
QT_MOC_LITERAL(147, 2207, 8), // "make_uri"
QT_MOC_LITERAL(148, 2216, 17), // "setCacheAttribute"
QT_MOC_LITERAL(149, 2234, 3), // "key"
QT_MOC_LITERAL(150, 2238, 3), // "val"
QT_MOC_LITERAL(151, 2242, 17), // "getCacheAttribute"
QT_MOC_LITERAL(152, 2260, 11), // "setUserNote"
QT_MOC_LITERAL(153, 2272, 4), // "note"
QT_MOC_LITERAL(154, 2277, 11), // "getUserNote"
QT_MOC_LITERAL(155, 2289, 8), // "getTxKey"
QT_MOC_LITERAL(156, 2298, 13), // "getTxKeyAsync"
QT_MOC_LITERAL(157, 2312, 10), // "checkTxKey"
QT_MOC_LITERAL(158, 2323, 6), // "tx_key"
QT_MOC_LITERAL(159, 2330, 10), // "getTxProof"
QT_MOC_LITERAL(160, 2341, 15), // "getTxProofAsync"
QT_MOC_LITERAL(161, 2357, 12), // "checkTxProof"
QT_MOC_LITERAL(162, 2370, 13), // "getSpendProof"
QT_MOC_LITERAL(163, 2384, 18), // "getSpendProofAsync"
QT_MOC_LITERAL(164, 2403, 15), // "checkSpendProof"
QT_MOC_LITERAL(165, 2419, 15), // "getReserveProof"
QT_MOC_LITERAL(166, 2435, 13), // "account_index"
QT_MOC_LITERAL(167, 2449, 17), // "checkReserveProof"
QT_MOC_LITERAL(168, 2467, 11), // "rescanSpent"
QT_MOC_LITERAL(169, 2479, 12), // "useForkRules"
QT_MOC_LITERAL(170, 2492, 7), // "version"
QT_MOC_LITERAL(171, 2500, 11), // "earlyBlocks"
QT_MOC_LITERAL(172, 2512, 7), // "getRing"
QT_MOC_LITERAL(173, 2520, 9), // "key_image"
QT_MOC_LITERAL(174, 2530, 8), // "getRings"
QT_MOC_LITERAL(175, 2539, 7), // "setRing"
QT_MOC_LITERAL(176, 2547, 4), // "ring"
QT_MOC_LITERAL(177, 2552, 8), // "relative"
QT_MOC_LITERAL(178, 2561, 23), // "segregatePreForkOutputs"
QT_MOC_LITERAL(179, 2585, 9), // "segregate"
QT_MOC_LITERAL(180, 2595, 17), // "segregationHeight"
QT_MOC_LITERAL(181, 2613, 19), // "keyReuseMitigation2"
QT_MOC_LITERAL(182, 2633, 10), // "mitigation"
QT_MOC_LITERAL(183, 2644, 19), // "onPassphraseEntered"
QT_MOC_LITERAL(184, 2664, 10), // "passphrase"
QT_MOC_LITERAL(185, 2675, 15), // "enter_on_device"
QT_MOC_LITERAL(186, 2691, 11), // "entry_abort"
QT_MOC_LITERAL(187, 2703, 12), // "disconnected"
QT_MOC_LITERAL(188, 2716, 10), // "refreshing"
QT_MOC_LITERAL(189, 2727, 4), // "seed"
QT_MOC_LITERAL(190, 2732, 12), // "seedLanguage"
QT_MOC_LITERAL(191, 2745, 6), // "Status"
QT_MOC_LITERAL(192, 2752, 7), // "nettype"
QT_MOC_LITERAL(193, 2760, 17), // "NetworkType::Type"
QT_MOC_LITERAL(194, 2778, 24), // "currentSubaddressAccount"
QT_MOC_LITERAL(195, 2803, 12), // "synchronized"
QT_MOC_LITERAL(196, 2816, 11), // "errorString"
QT_MOC_LITERAL(197, 2828, 7), // "history"
QT_MOC_LITERAL(198, 2836, 19), // "TransactionHistory*"
QT_MOC_LITERAL(199, 2856, 12), // "historyModel"
QT_MOC_LITERAL(200, 2869, 34), // "TransactionHistorySortFilterM..."
QT_MOC_LITERAL(201, 2904, 16), // "addressBookModel"
QT_MOC_LITERAL(202, 2921, 17), // "AddressBookModel*"
QT_MOC_LITERAL(203, 2939, 11), // "addressBook"
QT_MOC_LITERAL(204, 2951, 12), // "AddressBook*"
QT_MOC_LITERAL(205, 2964, 15), // "subaddressModel"
QT_MOC_LITERAL(206, 2980, 16), // "SubaddressModel*"
QT_MOC_LITERAL(207, 2997, 10), // "subaddress"
QT_MOC_LITERAL(208, 3008, 11), // "Subaddress*"
QT_MOC_LITERAL(209, 3020, 22), // "subaddressAccountModel"
QT_MOC_LITERAL(210, 3043, 23), // "SubaddressAccountModel*"
QT_MOC_LITERAL(211, 3067, 17), // "subaddressAccount"
QT_MOC_LITERAL(212, 3085, 18), // "SubaddressAccount*"
QT_MOC_LITERAL(213, 3104, 13), // "secretViewKey"
QT_MOC_LITERAL(214, 3118, 13), // "publicViewKey"
QT_MOC_LITERAL(215, 3132, 14), // "secretSpendKey"
QT_MOC_LITERAL(216, 3147, 14), // "publicSpendKey"
QT_MOC_LITERAL(217, 3162, 13), // "daemonLogPath"
QT_MOC_LITERAL(218, 3176, 20), // "walletCreationHeight"
QT_MOC_LITERAL(219, 3197, 9), // "Status_Ok"
QT_MOC_LITERAL(220, 3207, 12), // "Status_Error"
QT_MOC_LITERAL(221, 3220, 15), // "Status_Critical"
QT_MOC_LITERAL(222, 3236, 26), // "ConnectionStatus_Connected"
QT_MOC_LITERAL(223, 3263, 29), // "ConnectionStatus_Disconnected"
QT_MOC_LITERAL(224, 3293, 29), // "ConnectionStatus_WrongVersion"
QT_MOC_LITERAL(225, 3323, 27), // "ConnectionStatus_Connecting"
QT_MOC_LITERAL(226, 3351, 18), // "BackgroundSync_Off"
QT_MOC_LITERAL(227, 3370, 28), // "BackgroundSync_ReusePassword"
QT_MOC_LITERAL(228, 3399, 29) // "BackgroundSync_CustomPassword"

    },
    "Wallet\0updated\0\0refreshed\0moneySpent\0"
    "txId\0amount\0moneyReceived\0"
    "unconfirmedMoneyReceived\0newBlock\0"
    "height\0targetHeight\0backgroundSyncSetup\0"
    "backgroundSyncStarted\0backgroundSyncStopped\0"
    "addressBookChanged\0historyModelChanged\0"
    "walletCreationHeightChanged\0"
    "deviceButtonRequest\0buttonCode\0"
    "deviceButtonPressed\0walletPassphraseNeeded\0"
    "onDevice\0transactionCommitted\0status\0"
    "PendingTransaction*\0t\0txid\0heightRefreshed\0"
    "walletHeight\0daemonHeight\0"
    "deviceShowAddressShowed\0transactionCreated\0"
    "transaction\0QVector<QString>\0addresses\0"
    "paymentId\0mixinCount\0connectionStatusChanged\0"
    "currentSubaddressAccountChanged\0"
    "disconnectedChanged\0proxyAddressChanged\0"
    "refreshingChanged\0setSeedLanguage\0"
    "lang\0connected\0ConnectionStatus\0"
    "forceCheck\0setPassword\0password\0address\0"
    "accountIndex\0addressIndex\0storeAsync\0"
    "QJSValue\0callback\0path\0initAsync\0"
    "daemonAddress\0trustedDaemon\0"
    "upperTransactionLimit\0isRecovering\0"
    "isRecoveringFromDevice\0restoreHeight\0"
    "proxyAddress\0setDaemonLogin\0daemonUsername\0"
    "daemonPassword\0createViewOnly\0"
    "connectToDaemon\0setTrustedDaemon\0arg\0"
    "balance\0balanceAll\0unlockedBalance\0"
    "unlockedBalanceAll\0switchSubaddressAccount\0"
    "addSubaddressAccount\0label\0"
    "numSubaddressAccounts\0numSubaddresses\0"
    "addSubaddress\0getSubaddressLabel\0"
    "setSubaddressLabel\0deviceShowAddressAsync\0"
    "isHwBacked\0isLedger\0isTrezor\0viewOnly\0"
    "refreshHeightAsync\0exportKeyImages\0"
    "all\0importKeyImages\0exportOutputs\0"
    "importOutputs\0scanTransactions\0txids\0"
    "setupBackgroundSync\0BackgroundSyncType\0"
    "background_sync_type\0wallet_password\0"
    "getBackgroundSyncType\0isBackgroundWallet\0"
    "isBackgroundSyncing\0startBackgroundSync\0"
    "stopBackgroundSync\0refresh\0"
    "historyAndSubaddresses\0startRefresh\0"
    "pauseRefresh\0createTransactionAsync\0"
    "destinationAddresses\0payment_id\0"
    "destinationAmounts\0mixin_count\0"
    "PendingTransaction::Priority\0priority\0"
    "createTransactionAll\0dst_addr\0"
    "createTransactionAllAsync\0"
    "createSweepUnmixableTransaction\0"
    "createSweepUnmixableTransactionAsync\0"
    "loadTxFile\0UnsignedTransaction*\0"
    "fileName\0submitTxFile\0commitTransactionAsync\0"
    "disposeTransaction\0estimateTransactionFeeAsync\0"
    "QVector<quint64>\0amounts\0generatePaymentId\0"
    "integratedAddress\0signMessage\0message\0"
    "filename\0verifySignedMessage\0signature\0"
    "parse_uri\0uri\0QString&\0uint64_t&\0"
    "tx_description\0recipient_name\0"
    "QVector<QString>&\0unknown_parameters\0"
    "error\0make_uri\0setCacheAttribute\0key\0"
    "val\0getCacheAttribute\0setUserNote\0"
    "note\0getUserNote\0getTxKey\0getTxKeyAsync\0"
    "checkTxKey\0tx_key\0getTxProof\0"
    "getTxProofAsync\0checkTxProof\0getSpendProof\0"
    "getSpendProofAsync\0checkSpendProof\0"
    "getReserveProof\0account_index\0"
    "checkReserveProof\0rescanSpent\0"
    "useForkRules\0version\0earlyBlocks\0"
    "getRing\0key_image\0getRings\0setRing\0"
    "ring\0relative\0segregatePreForkOutputs\0"
    "segregate\0segregationHeight\0"
    "keyReuseMitigation2\0mitigation\0"
    "onPassphraseEntered\0passphrase\0"
    "enter_on_device\0entry_abort\0disconnected\0"
    "refreshing\0seed\0seedLanguage\0Status\0"
    "nettype\0NetworkType::Type\0"
    "currentSubaddressAccount\0synchronized\0"
    "errorString\0history\0TransactionHistory*\0"
    "historyModel\0TransactionHistorySortFilterModel*\0"
    "addressBookModel\0AddressBookModel*\0"
    "addressBook\0AddressBook*\0subaddressModel\0"
    "SubaddressModel*\0subaddress\0Subaddress*\0"
    "subaddressAccountModel\0SubaddressAccountModel*\0"
    "subaddressAccount\0SubaddressAccount*\0"
    "secretViewKey\0publicViewKey\0secretSpendKey\0"
    "publicSpendKey\0daemonLogPath\0"
    "walletCreationHeight\0Status_Ok\0"
    "Status_Error\0Status_Critical\0"
    "ConnectionStatus_Connected\0"
    "ConnectionStatus_Disconnected\0"
    "ConnectionStatus_WrongVersion\0"
    "ConnectionStatus_Connecting\0"
    "BackgroundSync_Off\0BackgroundSync_ReusePassword\0"
    "BackgroundSync_CustomPassword"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Wallet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     128,   14, // methods
      26, 1172, // properties
       3, 1276, // enums/sets
       0,    0, // constructors
       0,       // flags
      24,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  654,    2, 0x06 /* Public */,
       3,    0,  655,    2, 0x06 /* Public */,
       4,    2,  656,    2, 0x06 /* Public */,
       7,    2,  661,    2, 0x06 /* Public */,
       8,    2,  666,    2, 0x06 /* Public */,
       9,    2,  671,    2, 0x06 /* Public */,
      12,    0,  676,    2, 0x06 /* Public */,
      13,    0,  677,    2, 0x06 /* Public */,
      14,    0,  678,    2, 0x06 /* Public */,
      15,    0,  679,    2, 0x06 /* Public */,
      16,    0,  680,    2, 0x06 /* Public */,
      17,    0,  681,    2, 0x06 /* Public */,
      18,    1,  682,    2, 0x06 /* Public */,
      20,    0,  685,    2, 0x06 /* Public */,
      21,    1,  686,    2, 0x06 /* Public */,
      23,    3,  689,    2, 0x06 /* Public */,
      28,    3,  696,    2, 0x06 /* Public */,
      31,    0,  703,    2, 0x06 /* Public */,
      32,    4,  704,    2, 0x06 /* Public */,
      38,    1,  713,    2, 0x06 /* Public */,
      39,    0,  716,    2, 0x06 /* Public */,
      40,    0,  717,    2, 0x06 /* Public */,
      41,    0,  718,    2, 0x06 /* Public */,
      42,    0,  719,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      43,    1,  720,    2, 0x02 /* Public */,
      45,    1,  723,    2, 0x02 /* Public */,
      45,    0,  726,    2, 0x22 /* Public | MethodCloned */,
      48,    1,  727,    2, 0x02 /* Public */,
      50,    2,  730,    2, 0x02 /* Public */,
      53,    2,  735,    2, 0x02 /* Public */,
      53,    1,  740,    2, 0x22 /* Public | MethodCloned */,
      57,    7,  743,    2, 0x02 /* Public */,
      57,    6,  758,    2, 0x22 /* Public | MethodCloned */,
      57,    5,  771,    2, 0x22 /* Public | MethodCloned */,
      57,    4,  782,    2, 0x22 /* Public | MethodCloned */,
      57,    3,  791,    2, 0x22 /* Public | MethodCloned */,
      57,    2,  798,    2, 0x22 /* Public | MethodCloned */,
      57,    1,  803,    2, 0x22 /* Public | MethodCloned */,
      65,    2,  806,    2, 0x02 /* Public */,
      65,    1,  811,    2, 0x22 /* Public | MethodCloned */,
      65,    0,  814,    2, 0x22 /* Public | MethodCloned */,
      68,    2,  815,    2, 0x02 /* Public */,
      69,    0,  820,    2, 0x02 /* Public */,
      70,    1,  821,    2, 0x02 /* Public */,
      72,    0,  824,    2, 0x02 /* Public */,
      72,    1,  825,    2, 0x02 /* Public */,
      73,    0,  828,    2, 0x02 /* Public */,
      74,    0,  829,    2, 0x02 /* Public */,
      74,    1,  830,    2, 0x02 /* Public */,
      75,    0,  833,    2, 0x02 /* Public */,
      76,    1,  834,    2, 0x02 /* Public */,
      77,    1,  837,    2, 0x02 /* Public */,
      79,    0,  840,    2, 0x02 /* Public */,
      80,    1,  841,    2, 0x02 /* Public */,
      81,    1,  844,    2, 0x02 /* Public */,
      82,    2,  847,    2, 0x02 /* Public */,
      83,    3,  852,    2, 0x02 /* Public */,
      84,    3,  859,    2, 0x02 /* Public */,
      85,    0,  866,    2, 0x02 /* Public */,
      86,    0,  867,    2, 0x02 /* Public */,
      87,    0,  868,    2, 0x02 /* Public */,
      88,    0,  869,    2, 0x02 /* Public */,
      89,    0,  870,    2, 0x02 /* Public */,
      90,    2,  871,    2, 0x02 /* Public */,
      90,    1,  876,    2, 0x22 /* Public | MethodCloned */,
      92,    1,  879,    2, 0x02 /* Public */,
      93,    2,  882,    2, 0x02 /* Public */,
      93,    1,  887,    2, 0x22 /* Public | MethodCloned */,
      94,    1,  890,    2, 0x02 /* Public */,
      95,    1,  893,    2, 0x02 /* Public */,
      97,    2,  896,    2, 0x02 /* Public */,
     101,    0,  901,    2, 0x02 /* Public */,
     102,    0,  902,    2, 0x02 /* Public */,
     103,    0,  903,    2, 0x02 /* Public */,
     104,    0,  904,    2, 0x02 /* Public */,
     105,    1,  905,    2, 0x02 /* Public */,
     106,    1,  908,    2, 0x02 /* Public */,
     106,    0,  911,    2, 0x22 /* Public | MethodCloned */,
     108,    0,  912,    2, 0x02 /* Public */,
     109,    0,  913,    2, 0x02 /* Public */,
     110,    5,  914,    2, 0x02 /* Public */,
     117,    4,  925,    2, 0x02 /* Public */,
     119,    4,  934,    2, 0x02 /* Public */,
     120,    0,  943,    2, 0x02 /* Public */,
     121,    0,  944,    2, 0x02 /* Public */,
     122,    1,  945,    2, 0x02 /* Public */,
     125,    1,  948,    2, 0x02 /* Public */,
     126,    1,  951,    2, 0x02 /* Public */,
     127,    1,  954,    2, 0x02 /* Public */,
     127,    1,  957,    2, 0x02 /* Public */,
     128,    4,  960,    2, 0x02 /* Public */,
     131,    0,  969,    2, 0x02 /* Public */,
     132,    1,  970,    2, 0x02 /* Public */,
     133,    2,  973,    2, 0x02 /* Public */,
     133,    1,  978,    2, 0x22 /* Public | MethodCloned */,
     136,    4,  981,    2, 0x02 /* Public */,
     136,    3,  990,    2, 0x22 /* Public | MethodCloned */,
     138,    8,  997,    2, 0x02 /* Public */,
     147,    4, 1014,    2, 0x02 /* Public */,
     147,    3, 1023,    2, 0x22 /* Public | MethodCloned */,
     147,    2, 1030,    2, 0x22 /* Public | MethodCloned */,
     147,    1, 1035,    2, 0x22 /* Public | MethodCloned */,
     148,    2, 1038,    2, 0x02 /* Public */,
     151,    1, 1043,    2, 0x02 /* Public */,
     152,    2, 1046,    2, 0x02 /* Public */,
     154,    1, 1051,    2, 0x02 /* Public */,
     155,    1, 1054,    2, 0x02 /* Public */,
     156,    2, 1057,    2, 0x02 /* Public */,
     157,    3, 1062,    2, 0x02 /* Public */,
     159,    3, 1069,    2, 0x02 /* Public */,
     160,    4, 1076,    2, 0x02 /* Public */,
     161,    4, 1085,    2, 0x02 /* Public */,
     162,    2, 1094,    2, 0x02 /* Public */,
     163,    3, 1099,    2, 0x02 /* Public */,
     164,    3, 1106,    2, 0x02 /* Public */,
     165,    4, 1113,    2, 0x02 /* Public */,
     167,    3, 1122,    2, 0x02 /* Public */,
     168,    0, 1129,    2, 0x02 /* Public */,
     169,    2, 1130,    2, 0x02 /* Public */,
     169,    1, 1135,    2, 0x22 /* Public | MethodCloned */,
     172,    1, 1138,    2, 0x02 /* Public */,
     174,    1, 1141,    2, 0x02 /* Public */,
     175,    3, 1144,    2, 0x02 /* Public */,
     178,    1, 1151,    2, 0x02 /* Public */,
     180,    1, 1154,    2, 0x02 /* Public */,
     181,    1, 1157,    2, 0x02 /* Public */,
     183,    3, 1160,    2, 0x02 /* Public */,
     183,    2, 1167,    2, 0x22 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::ULongLong,    5,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::ULongLong,    5,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::ULongLong,    5,    6,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::ULongLong,   10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::ULongLong,   19,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 25, QMetaType::QStringList,   24,   26,   27,
    QMetaType::Void, QMetaType::ULongLong, QMetaType::ULongLong, QMetaType::ULongLong,   29,   30,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 25, 0x80000000 | 34, QMetaType::QString, QMetaType::UInt,   33,   35,   36,   37,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::QString,   44,
    0x80000000 | 46, QMetaType::Bool,   47,
    0x80000000 | 46,
    QMetaType::Bool, QMetaType::QString,   49,
    QMetaType::QString, QMetaType::UInt, QMetaType::UInt,   51,   52,
    QMetaType::Void, 0x80000000 | 54, QMetaType::QString,   55,   56,
    QMetaType::Void, 0x80000000 | 54,   55,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::ULongLong, QMetaType::Bool, QMetaType::Bool, QMetaType::ULongLong, QMetaType::QString,   58,   59,   60,   61,   62,   63,   64,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::ULongLong, QMetaType::Bool, QMetaType::Bool, QMetaType::ULongLong,   58,   59,   60,   61,   62,   63,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::ULongLong, QMetaType::Bool, QMetaType::Bool,   58,   59,   60,   61,   62,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::ULongLong, QMetaType::Bool,   58,   59,   60,   61,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::ULongLong,   58,   59,   60,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   58,   59,
    QMetaType::Void, QMetaType::QString,   58,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   66,   67,
    QMetaType::Void, QMetaType::QString,   66,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   56,   49,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   71,
    QMetaType::ULongLong,
    QMetaType::ULongLong, QMetaType::UInt,   51,
    QMetaType::ULongLong,
    QMetaType::ULongLong,
    QMetaType::ULongLong, QMetaType::UInt,   51,
    QMetaType::ULongLong,
    QMetaType::Void, QMetaType::UInt,   51,
    QMetaType::Void, QMetaType::QString,   78,
    QMetaType::UInt,
    QMetaType::UInt, QMetaType::UInt,   51,
    QMetaType::Void, QMetaType::QString,   78,
    QMetaType::QString, QMetaType::UInt, QMetaType::UInt,   51,   52,
    QMetaType::Void, QMetaType::UInt, QMetaType::UInt, QMetaType::QString,   51,   52,   78,
    QMetaType::Void, QMetaType::UInt, QMetaType::UInt, QMetaType::QString,   51,   52,   36,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool,   56,   91,
    QMetaType::Bool, QMetaType::QString,   56,
    QMetaType::Bool, QMetaType::QString,   56,
    QMetaType::Bool, QMetaType::QString, QMetaType::Bool,   56,   91,
    QMetaType::Bool, QMetaType::QString,   56,
    QMetaType::Bool, QMetaType::QString,   56,
    QMetaType::Bool, 0x80000000 | 34,   96,
    QMetaType::Void, 0x80000000 | 98, QMetaType::QString,   99,  100,
    0x80000000 | 98,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Bool, QMetaType::Bool,  107,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 34, QMetaType::QString, 0x80000000 | 34, QMetaType::UInt, 0x80000000 | 115,  111,  112,  113,  114,  116,
    0x80000000 | 25, QMetaType::QString, QMetaType::QString, QMetaType::UInt, 0x80000000 | 115,  118,  112,  114,  116,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::UInt, 0x80000000 | 115,  118,  112,  114,  116,
    0x80000000 | 25,
    QMetaType::Void,
    0x80000000 | 123, QMetaType::QString,  124,
    QMetaType::Bool, QMetaType::QString,  124,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, 0x80000000 | 123,   26,
    QMetaType::Void, 0x80000000 | 34, 0x80000000 | 129, 0x80000000 | 115, 0x80000000 | 54,  111,  130,  116,   55,
    QMetaType::QString,
    QMetaType::QString, QMetaType::QString,   36,
    QMetaType::QString, QMetaType::QString, QMetaType::Bool,  134,  135,
    QMetaType::QString, QMetaType::QString,  134,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Bool,  134,   50,  137,  135,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QString,  134,   50,  137,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 140, 0x80000000 | 140, 0x80000000 | 141, 0x80000000 | 140, 0x80000000 | 140, 0x80000000 | 144, 0x80000000 | 140,  139,   50,  112,    6,  142,  143,  145,  146,
    QMetaType::QString, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString, QMetaType::QString,   50,    6,  142,  143,
    QMetaType::QString, QMetaType::QString, QMetaType::ULongLong, QMetaType::QString,   50,    6,  142,
    QMetaType::QString, QMetaType::QString, QMetaType::ULongLong,   50,    6,
    QMetaType::QString, QMetaType::QString,   50,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,  149,  150,
    QMetaType::QString, QMetaType::QString,  149,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   27,  153,
    QMetaType::QString, QMetaType::QString,   27,
    QMetaType::QString, QMetaType::QString,   27,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 54,   27,   55,
    QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   27,  158,   50,
    QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   27,   50,  134,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 54,   27,   50,  134,   55,
    QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   27,   50,  134,  137,
    QMetaType::QString, QMetaType::QString, QMetaType::QString,   27,  134,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 54,   27,  134,   55,
    QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   27,  134,  137,
    QMetaType::QString, QMetaType::Bool, QMetaType::UInt, QMetaType::ULongLong, QMetaType::QString,   91,  166,    6,  134,
    QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   50,  134,  137,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::UChar, QMetaType::ULongLong,  170,  171,
    QMetaType::Bool, QMetaType::UChar,  170,
    QMetaType::QString, QMetaType::QString,  173,
    QMetaType::QString, QMetaType::QString,   27,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::Bool,  173,  176,  177,
    QMetaType::Void, QMetaType::Bool,  179,
    QMetaType::Void, QMetaType::ULongLong,   10,
    QMetaType::Void, QMetaType::Bool,  182,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool, QMetaType::Bool,  184,  185,  186,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,  184,  185,

 // properties: name, type, flags
     187, QMetaType::Bool, 0x00495001,
     188, QMetaType::Bool, 0x00495001,
     189, QMetaType::QString, 0x00095001,
     190, QMetaType::QString, 0x00095001,
      24, 0x80000000 | 191, 0x00095009,
     192, 0x80000000 | 193, 0x00095009,
     194, QMetaType::UInt, 0x00495001,
     195, QMetaType::Bool, 0x00095001,
     196, QMetaType::QString, 0x00095001,
     197, 0x80000000 | 198, 0x00095009,
     199, 0x80000000 | 200, 0x00495009,
      56, QMetaType::QString, 0x00095001,
     201, 0x80000000 | 202, 0x00095009,
     203, 0x80000000 | 204, 0x00495009,
     205, 0x80000000 | 206, 0x00095009,
     207, 0x80000000 | 208, 0x00095009,
     209, 0x80000000 | 210, 0x00095009,
     211, 0x80000000 | 212, 0x00095009,
      88, QMetaType::Bool, 0x00095001,
     213, QMetaType::QString, 0x00095001,
     214, QMetaType::QString, 0x00095001,
     215, QMetaType::QString, 0x00095001,
     216, QMetaType::QString, 0x00095001,
     217, QMetaType::QString, 0x00095401,
      64, QMetaType::QString, 0x00495103,
     218, QMetaType::ULongLong, 0x00495103,

 // properties: notify_signal_id
      21,
      23,
       0,
       0,
       0,
       0,
      20,
       0,
       0,
       0,
      10,
       0,
       0,
       9,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      22,
      11,

 // enums: name, alias, flags, count, data
     191,  191, 0x0,    3, 1291,
      46,   46, 0x0,    4, 1297,
      98,   98, 0x0,    3, 1305,

 // enum data: key, value
     219, uint(Wallet::Status_Ok),
     220, uint(Wallet::Status_Error),
     221, uint(Wallet::Status_Critical),
     222, uint(Wallet::ConnectionStatus_Connected),
     223, uint(Wallet::ConnectionStatus_Disconnected),
     224, uint(Wallet::ConnectionStatus_WrongVersion),
     225, uint(Wallet::ConnectionStatus_Connecting),
     226, uint(Wallet::BackgroundSync_Off),
     227, uint(Wallet::BackgroundSync_ReusePassword),
     228, uint(Wallet::BackgroundSync_CustomPassword),

       0        // eod
};

void Wallet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Wallet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updated(); break;
        case 1: _t->refreshed(); break;
        case 2: _t->moneySpent((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2]))); break;
        case 3: _t->moneyReceived((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2]))); break;
        case 4: _t->unconfirmedMoneyReceived((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2]))); break;
        case 5: _t->newBlock((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2]))); break;
        case 6: _t->backgroundSyncSetup(); break;
        case 7: _t->backgroundSyncStarted(); break;
        case 8: _t->backgroundSyncStopped(); break;
        case 9: _t->addressBookChanged(); break;
        case 10: _t->historyModelChanged(); break;
        case 11: _t->walletCreationHeightChanged(); break;
        case 12: _t->deviceButtonRequest((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 13: _t->deviceButtonPressed(); break;
        case 14: _t->walletPassphraseNeeded((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->transactionCommitted((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< PendingTransaction*(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 16: _t->heightRefreshed((*reinterpret_cast< quint64(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3]))); break;
        case 17: _t->deviceShowAddressShowed(); break;
        case 18: _t->transactionCreated((*reinterpret_cast< PendingTransaction*(*)>(_a[1])),(*reinterpret_cast< const QVector<QString>(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< quint32(*)>(_a[4]))); break;
        case 19: _t->connectionStatusChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->currentSubaddressAccountChanged(); break;
        case 21: _t->disconnectedChanged(); break;
        case 22: _t->proxyAddressChanged(); break;
        case 23: _t->refreshingChanged(); break;
        case 24: _t->setSeedLanguage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 25: { ConnectionStatus _r = _t->connected((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< ConnectionStatus*>(_a[0]) = std::move(_r); }  break;
        case 26: { ConnectionStatus _r = _t->connected();
            if (_a[0]) *reinterpret_cast< ConnectionStatus*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->setPassword((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { QString _r = _t->address((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->storeAsync((*reinterpret_cast< const QJSValue(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 30: _t->storeAsync((*reinterpret_cast< const QJSValue(*)>(_a[1]))); break;
        case 31: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< quint64(*)>(_a[6])),(*reinterpret_cast< const QString(*)>(_a[7]))); break;
        case 32: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< quint64(*)>(_a[6]))); break;
        case 33: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5]))); break;
        case 34: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 35: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3]))); break;
        case 36: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 37: _t->initAsync((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 38: _t->setDaemonLogin((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 39: _t->setDaemonLogin((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 40: _t->setDaemonLogin(); break;
        case 41: { bool _r = _t->createViewOnly((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 42: { bool _r = _t->connectToDaemon();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->setTrustedDaemon((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 44: { quint64 _r = _t->balance();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 45: { quint64 _r = _t->balance((*reinterpret_cast< quint32(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 46: { quint64 _r = _t->balanceAll();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 47: { quint64 _r = _t->unlockedBalance();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 48: { quint64 _r = _t->unlockedBalance((*reinterpret_cast< quint32(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 49: { quint64 _r = _t->unlockedBalanceAll();
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->switchSubaddressAccount((*reinterpret_cast< quint32(*)>(_a[1]))); break;
        case 51: _t->addSubaddressAccount((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 52: { quint32 _r = _t->numSubaddressAccounts();
            if (_a[0]) *reinterpret_cast< quint32*>(_a[0]) = std::move(_r); }  break;
        case 53: { quint32 _r = _t->numSubaddresses((*reinterpret_cast< quint32(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint32*>(_a[0]) = std::move(_r); }  break;
        case 54: _t->addSubaddress((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 55: { QString _r = _t->getSubaddressLabel((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 56: _t->setSubaddressLabel((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 57: _t->deviceShowAddressAsync((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 58: { bool _r = _t->isHwBacked();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 59: { bool _r = _t->isLedger();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 60: { bool _r = _t->isTrezor();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 61: { bool _r = _t->viewOnly();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 62: _t->refreshHeightAsync(); break;
        case 63: { bool _r = _t->exportKeyImages((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 64: { bool _r = _t->exportKeyImages((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 65: { bool _r = _t->importKeyImages((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 66: { bool _r = _t->exportOutputs((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 67: { bool _r = _t->exportOutputs((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 68: { bool _r = _t->importOutputs((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 69: { bool _r = _t->scanTransactions((*reinterpret_cast< const QVector<QString>(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 70: _t->setupBackgroundSync((*reinterpret_cast< const BackgroundSyncType(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 71: { BackgroundSyncType _r = _t->getBackgroundSyncType();
            if (_a[0]) *reinterpret_cast< BackgroundSyncType*>(_a[0]) = std::move(_r); }  break;
        case 72: { bool _r = _t->isBackgroundWallet();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 73: { bool _r = _t->isBackgroundSyncing();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 74: _t->startBackgroundSync(); break;
        case 75: _t->stopBackgroundSync((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 76: { bool _r = _t->refresh((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 77: { bool _r = _t->refresh();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 78: _t->startRefresh(); break;
        case 79: _t->pauseRefresh(); break;
        case 80: _t->createTransactionAsync((*reinterpret_cast< const QVector<QString>(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVector<QString>(*)>(_a[3])),(*reinterpret_cast< quint32(*)>(_a[4])),(*reinterpret_cast< PendingTransaction::Priority(*)>(_a[5]))); break;
        case 81: { PendingTransaction* _r = _t->createTransactionAll((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< quint32(*)>(_a[3])),(*reinterpret_cast< PendingTransaction::Priority(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< PendingTransaction**>(_a[0]) = std::move(_r); }  break;
        case 82: _t->createTransactionAllAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< quint32(*)>(_a[3])),(*reinterpret_cast< PendingTransaction::Priority(*)>(_a[4]))); break;
        case 83: { PendingTransaction* _r = _t->createSweepUnmixableTransaction();
            if (_a[0]) *reinterpret_cast< PendingTransaction**>(_a[0]) = std::move(_r); }  break;
        case 84: _t->createSweepUnmixableTransactionAsync(); break;
        case 85: { UnsignedTransaction* _r = _t->loadTxFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< UnsignedTransaction**>(_a[0]) = std::move(_r); }  break;
        case 86: { bool _r = _t->submitTxFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 87: _t->commitTransactionAsync((*reinterpret_cast< PendingTransaction*(*)>(_a[1]))); break;
        case 88: _t->disposeTransaction((*reinterpret_cast< PendingTransaction*(*)>(_a[1]))); break;
        case 89: _t->disposeTransaction((*reinterpret_cast< UnsignedTransaction*(*)>(_a[1]))); break;
        case 90: _t->estimateTransactionFeeAsync((*reinterpret_cast< const QVector<QString>(*)>(_a[1])),(*reinterpret_cast< const QVector<quint64>(*)>(_a[2])),(*reinterpret_cast< PendingTransaction::Priority(*)>(_a[3])),(*reinterpret_cast< const QJSValue(*)>(_a[4]))); break;
        case 91: { QString _r = _t->generatePaymentId();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 92: { QString _r = _t->integratedAddress((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 93: { QString _r = _t->signMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 94: { QString _r = _t->signMessage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 95: { bool _r = _t->verifySignedMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 96: { bool _r = _t->verifySignedMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 97: { bool _r = _t->parse_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3])),(*reinterpret_cast< uint64_t(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6])),(*reinterpret_cast< QVector<QString>(*)>(_a[7])),(*reinterpret_cast< QString(*)>(_a[8])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 98: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const quint64(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 99: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const quint64(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 100: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const quint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 101: { QString _r = _t->make_uri((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 102: { bool _r = _t->setCacheAttribute((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 103: { QString _r = _t->getCacheAttribute((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 104: { bool _r = _t->setUserNote((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 105: { QString _r = _t->getUserNote((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 106: { QString _r = _t->getTxKey((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 107: _t->getTxKeyAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QJSValue(*)>(_a[2]))); break;
        case 108: { QString _r = _t->checkTxKey((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 109: { QString _r = _t->getTxProof((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 110: _t->getTxProofAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QJSValue(*)>(_a[4]))); break;
        case 111: { QString _r = _t->checkTxProof((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 112: { QString _r = _t->getSpendProof((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 113: _t->getSpendProofAsync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QJSValue(*)>(_a[3]))); break;
        case 114: { QString _r = _t->checkSpendProof((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 115: { QString _r = _t->getReserveProof((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< quint64(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 116: { QString _r = _t->checkReserveProof((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 117: { bool _r = _t->rescanSpent();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 118: { bool _r = _t->useForkRules((*reinterpret_cast< quint8(*)>(_a[1])),(*reinterpret_cast< quint64(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 119: { bool _r = _t->useForkRules((*reinterpret_cast< quint8(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 120: { QString _r = _t->getRing((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 121: { QString _r = _t->getRings((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 122: { bool _r = _t->setRing((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 123: _t->segregatePreForkOutputs((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 124: _t->segregationHeight((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 125: _t->keyReuseMitigation2((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 126: _t->onPassphraseEntered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 127: _t->onPassphraseEntered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PendingTransaction* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PendingTransaction* >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<QString> >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 69:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<QString> >(); break;
            }
            break;
        case 80:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<QString> >(); break;
            }
            break;
        case 87:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PendingTransaction* >(); break;
            }
            break;
        case 88:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PendingTransaction* >(); break;
            }
            break;
        case 89:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< UnsignedTransaction* >(); break;
            }
            break;
        case 90:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<QString> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<quint64> >(); break;
            }
            break;
        case 107:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 110:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 113:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Wallet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::updated)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Wallet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::refreshed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(const QString & , quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::moneySpent)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(const QString & , quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::moneyReceived)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(const QString & , quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::unconfirmedMoneyReceived)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(quint64 , quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::newBlock)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::backgroundSyncSetup)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::backgroundSyncStarted)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::backgroundSyncStopped)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::addressBookChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::historyModelChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Wallet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::walletCreationHeightChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::deviceButtonRequest)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Wallet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::deviceButtonPressed)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::walletPassphraseNeeded)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(bool , PendingTransaction * , const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::transactionCommitted)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(quint64 , quint64 , quint64 ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::heightRefreshed)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (Wallet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::deviceShowAddressShowed)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(PendingTransaction * , const QVector<QString> & , const QString & , quint32 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::transactionCreated)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (Wallet::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::connectionStatusChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::currentSubaddressAccountChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::disconnectedChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::proxyAddressChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (Wallet::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Wallet::refreshingChanged)) {
                *result = 23;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Wallet *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->disconnected(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->refreshing(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->getSeed(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->getSeedLanguage(); break;
        case 4: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 5: *reinterpret_cast< NetworkType::Type*>(_v) = _t->nettype(); break;
        case 6: *reinterpret_cast< quint32*>(_v) = _t->currentSubaddressAccount(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->synchronized(); break;
        case 8: *reinterpret_cast< QString*>(_v) = _t->errorString(); break;
        case 9: *reinterpret_cast< TransactionHistory**>(_v) = _t->history(); break;
        case 10: *reinterpret_cast< TransactionHistorySortFilterModel**>(_v) = _t->historyModel(); break;
        case 11: *reinterpret_cast< QString*>(_v) = _t->path(); break;
        case 12: *reinterpret_cast< AddressBookModel**>(_v) = _t->addressBookModel(); break;
        case 13: *reinterpret_cast< AddressBook**>(_v) = _t->addressBook(); break;
        case 14: *reinterpret_cast< SubaddressModel**>(_v) = _t->subaddressModel(); break;
        case 15: *reinterpret_cast< Subaddress**>(_v) = _t->subaddress(); break;
        case 16: *reinterpret_cast< SubaddressAccountModel**>(_v) = _t->subaddressAccountModel(); break;
        case 17: *reinterpret_cast< SubaddressAccount**>(_v) = _t->subaddressAccount(); break;
        case 18: *reinterpret_cast< bool*>(_v) = _t->viewOnly(); break;
        case 19: *reinterpret_cast< QString*>(_v) = _t->getSecretViewKey(); break;
        case 20: *reinterpret_cast< QString*>(_v) = _t->getPublicViewKey(); break;
        case 21: *reinterpret_cast< QString*>(_v) = _t->getSecretSpendKey(); break;
        case 22: *reinterpret_cast< QString*>(_v) = _t->getPublicSpendKey(); break;
        case 23: *reinterpret_cast< QString*>(_v) = _t->getDaemonLogPath(); break;
        case 24: *reinterpret_cast< QString*>(_v) = _t->getProxyAddress(); break;
        case 25: *reinterpret_cast< quint64*>(_v) = _t->getWalletCreationHeight(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Wallet *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 24: _t->setProxyAddress(*reinterpret_cast< QString*>(_v)); break;
        case 25: _t->setWalletCreationHeight(*reinterpret_cast< quint64*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_Wallet[] = {
    QMetaObject::SuperData::link<NetworkType::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject Wallet::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Wallet.data,
    qt_meta_data_Wallet,
    qt_static_metacall,
    qt_meta_extradata_Wallet,
    nullptr
} };


const QMetaObject *Wallet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Wallet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Wallet.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PassprasePrompter"))
        return static_cast< PassprasePrompter*>(this);
    return QObject::qt_metacast(_clname);
}

int Wallet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 128)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 128;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 128)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 128;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 26;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Wallet::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Wallet::refreshed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Wallet::moneySpent(const QString & _t1, quint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Wallet::moneyReceived(const QString & _t1, quint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Wallet::unconfirmedMoneyReceived(const QString & _t1, quint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Wallet::newBlock(quint64 _t1, quint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Wallet::backgroundSyncSetup()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Wallet::backgroundSyncStarted()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Wallet::backgroundSyncStopped()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Wallet::addressBookChanged()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Wallet::historyModelChanged()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Wallet::walletCreationHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Wallet::deviceButtonRequest(quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Wallet::deviceButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Wallet::walletPassphraseNeeded(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void Wallet::transactionCommitted(bool _t1, PendingTransaction * _t2, const QStringList & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void Wallet::heightRefreshed(quint64 _t1, quint64 _t2, quint64 _t3)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 16, _a);
}

// SIGNAL 17
void Wallet::deviceShowAddressShowed()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void Wallet::transactionCreated(PendingTransaction * _t1, const QVector<QString> & _t2, const QString & _t3, quint32 _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void Wallet::connectionStatusChanged(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 19, _a);
}

// SIGNAL 20
void Wallet::currentSubaddressAccountChanged()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void Wallet::disconnectedChanged()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void Wallet::proxyAddressChanged()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void Wallet::refreshingChanged()const
{
    QMetaObject::activate(const_cast< Wallet *>(this), &staticMetaObject, 23, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
