/****************************************************************************
** Meta object code from reading C++ file 'AddressBook.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/libwalletqt/AddressBook.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AddressBook.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AddressBook_t {
    QByteArrayData data[24];
    char stringdata0[292];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AddressBook_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AddressBook_t qt_meta_stringdata_AddressBook = {
    {
QT_MOC_LITERAL(0, 0, 11), // "AddressBook"
QT_MOC_LITERAL(1, 12, 14), // "refreshStarted"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 15), // "refreshFinished"
QT_MOC_LITERAL(4, 44, 6), // "getRow"
QT_MOC_LITERAL(5, 51, 5), // "index"
QT_MOC_LITERAL(6, 57, 44), // "std::function<void(Monero::Ad..."
QT_MOC_LITERAL(7, 102, 8), // "callback"
QT_MOC_LITERAL(8, 111, 6), // "addRow"
QT_MOC_LITERAL(9, 118, 7), // "address"
QT_MOC_LITERAL(10, 126, 10), // "payment_id"
QT_MOC_LITERAL(11, 137, 11), // "description"
QT_MOC_LITERAL(12, 149, 9), // "deleteRow"
QT_MOC_LITERAL(13, 159, 5), // "rowId"
QT_MOC_LITERAL(14, 165, 11), // "errorString"
QT_MOC_LITERAL(15, 177, 9), // "errorCode"
QT_MOC_LITERAL(16, 187, 14), // "getDescription"
QT_MOC_LITERAL(17, 202, 14), // "setDescription"
QT_MOC_LITERAL(18, 217, 5), // "label"
QT_MOC_LITERAL(19, 223, 9), // "ErrorCode"
QT_MOC_LITERAL(20, 233, 9), // "Status_Ok"
QT_MOC_LITERAL(21, 243, 13), // "General_Error"
QT_MOC_LITERAL(22, 257, 15), // "Invalid_Address"
QT_MOC_LITERAL(23, 273, 18) // "Invalid_Payment_Id"

    },
    "AddressBook\0refreshStarted\0\0refreshFinished\0"
    "getRow\0index\0std::function<void(Monero::AddressBookRow&)>\0"
    "callback\0addRow\0address\0payment_id\0"
    "description\0deleteRow\0rowId\0errorString\0"
    "errorCode\0getDescription\0setDescription\0"
    "label\0ErrorCode\0Status_Ok\0General_Error\0"
    "Invalid_Address\0Invalid_Payment_Id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AddressBook[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       1,   86, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    0,   60,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       4,    2,   61,    2, 0x02 /* Public */,
       8,    3,   66,    2, 0x02 /* Public */,
      12,    1,   73,    2, 0x02 /* Public */,
      14,    0,   76,    2, 0x02 /* Public */,
      15,    0,   77,    2, 0x02 /* Public */,
      16,    1,   78,    2, 0x02 /* Public */,
      17,    2,   81,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 6,    5,    7,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QString,    9,   10,   11,
    QMetaType::Bool, QMetaType::Int,   13,
    QMetaType::QString,
    QMetaType::Int,
    QMetaType::QString, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    5,   18,

 // enums: name, alias, flags, count, data
      19,   19, 0x0,    4,   91,

 // enum data: key, value
      20, uint(AddressBook::Status_Ok),
      21, uint(AddressBook::General_Error),
      22, uint(AddressBook::Invalid_Address),
      23, uint(AddressBook::Invalid_Payment_Id),

       0        // eod
};

void AddressBook::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressBook *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refreshStarted(); break;
        case 1: _t->refreshFinished(); break;
        case 2: { bool _r = _t->getRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< std::function<void(Monero::AddressBookRow&)>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->addRow((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->deleteRow((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->errorCode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->getDescription((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setDescription((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AddressBook::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBook::refreshStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AddressBook::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressBook::refreshFinished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AddressBook::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_AddressBook.data,
    qt_meta_data_AddressBook,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AddressBook::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AddressBook::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AddressBook.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AddressBook::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void AddressBook::refreshStarted()const
{
    QMetaObject::activate(const_cast< AddressBook *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AddressBook::refreshFinished()const
{
    QMetaObject::activate(const_cast< AddressBook *>(this), &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
