/****************************************************************************
** Meta object code from reading C++ file 'SubaddressAccount.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/libwalletqt/SubaddressAccount.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SubaddressAccount.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SubaddressAccount_t {
    QByteArrayData data[14];
    char stringdata0[173];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SubaddressAccount_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SubaddressAccount_t qt_meta_stringdata_SubaddressAccount = {
    {
QT_MOC_LITERAL(0, 0, 17), // "SubaddressAccount"
QT_MOC_LITERAL(1, 18, 14), // "refreshStarted"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 15), // "refreshFinished"
QT_MOC_LITERAL(4, 50, 6), // "getAll"
QT_MOC_LITERAL(5, 57, 6), // "getRow"
QT_MOC_LITERAL(6, 64, 5), // "index"
QT_MOC_LITERAL(7, 70, 50), // "std::function<void(Monero::Su..."
QT_MOC_LITERAL(8, 121, 8), // "callback"
QT_MOC_LITERAL(9, 130, 6), // "addRow"
QT_MOC_LITERAL(10, 137, 5), // "label"
QT_MOC_LITERAL(11, 143, 8), // "setLabel"
QT_MOC_LITERAL(12, 152, 12), // "accountIndex"
QT_MOC_LITERAL(13, 165, 7) // "refresh"

    },
    "SubaddressAccount\0refreshStarted\0\0"
    "refreshFinished\0getAll\0getRow\0index\0"
    "std::function<void(Monero::SubaddressAccountRow&)>\0"
    "callback\0addRow\0label\0setLabel\0"
    "accountIndex\0refresh"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SubaddressAccount[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       4,    0,   51,    2, 0x02 /* Public */,
       5,    2,   52,    2, 0x02 /* Public */,
       9,    1,   57,    2, 0x02 /* Public */,
      11,    2,   60,    2, 0x02 /* Public */,
      13,    0,   65,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Bool, QMetaType::Int, 0x80000000 | 7,    6,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::UInt, QMetaType::QString,   12,   10,
    QMetaType::Void,

       0        // eod
};

void SubaddressAccount::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SubaddressAccount *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refreshStarted(); break;
        case 1: _t->refreshFinished(); break;
        case 2: _t->getAll(); break;
        case 3: { bool _r = _t->getRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< std::function<void(Monero::SubaddressAccountRow&)>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->addRow((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->setLabel((*reinterpret_cast< quint32(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->refresh(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SubaddressAccount::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SubaddressAccount::refreshStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SubaddressAccount::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SubaddressAccount::refreshFinished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SubaddressAccount::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_SubaddressAccount.data,
    qt_meta_data_SubaddressAccount,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SubaddressAccount::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SubaddressAccount::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SubaddressAccount.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SubaddressAccount::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void SubaddressAccount::refreshStarted()const
{
    QMetaObject::activate(const_cast< SubaddressAccount *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SubaddressAccount::refreshFinished()const
{
    QMetaObject::activate(const_cast< SubaddressAccount *>(this), &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
