/****************************************************************************
** Meta object code from reading C++ file 'P2PoolManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/p2pool/P2PoolManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'P2PoolManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_P2PoolManager_t {
    QByteArrayData data[23];
    char stringdata0[280];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_P2PoolManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_P2PoolManager_t qt_meta_stringdata_P2PoolManager = {
    {
QT_MOC_LITERAL(0, 0, 13), // "P2PoolManager"
QT_MOC_LITERAL(1, 14, 18), // "p2poolStartFailure"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 12), // "p2poolStatus"
QT_MOC_LITERAL(4, 47, 8), // "isMining"
QT_MOC_LITERAL(5, 56, 8), // "hashrate"
QT_MOC_LITERAL(6, 65, 21), // "p2poolDownloadFailure"
QT_MOC_LITERAL(7, 87, 9), // "errorCode"
QT_MOC_LITERAL(8, 97, 21), // "p2poolDownloadSuccess"
QT_MOC_LITERAL(9, 119, 5), // "start"
QT_MOC_LITERAL(10, 125, 5), // "flags"
QT_MOC_LITERAL(11, 131, 7), // "address"
QT_MOC_LITERAL(12, 139, 5), // "chain"
QT_MOC_LITERAL(13, 145, 7), // "threads"
QT_MOC_LITERAL(14, 153, 4), // "exit"
QT_MOC_LITERAL(15, 158, 11), // "isInstalled"
QT_MOC_LITERAL(16, 170, 9), // "getStatus"
QT_MOC_LITERAL(17, 180, 8), // "download"
QT_MOC_LITERAL(18, 189, 13), // "DownloadError"
QT_MOC_LITERAL(19, 203, 18), // "BinaryNotAvailable"
QT_MOC_LITERAL(20, 222, 15), // "ConnectionIssue"
QT_MOC_LITERAL(21, 238, 22), // "HashVerificationFailed"
QT_MOC_LITERAL(22, 261, 18) // "InstallationFailed"

    },
    "P2PoolManager\0p2poolStartFailure\0\0"
    "p2poolStatus\0isMining\0hashrate\0"
    "p2poolDownloadFailure\0errorCode\0"
    "p2poolDownloadSuccess\0start\0flags\0"
    "address\0chain\0threads\0exit\0isInstalled\0"
    "getStatus\0download\0DownloadError\0"
    "BinaryNotAvailable\0ConnectionIssue\0"
    "HashVerificationFailed\0InstallationFailed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_P2PoolManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       1,   82, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    2,   60,    2, 0x06 /* Public */,
       6,    1,   65,    2, 0x06 /* Public */,
       8,    0,   68,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       9,    4,   69,    2, 0x02 /* Public */,
      14,    0,   78,    2, 0x02 /* Public */,
      15,    0,   79,    2, 0x02 /* Public */,
      16,    0,   80,    2, 0x02 /* Public */,
      17,    0,   81,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Int,    4,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   10,   11,   12,   13,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,

 // enums: name, alias, flags, count, data
      18,   18, 0x0,    4,   87,

 // enum data: key, value
      19, uint(P2PoolManager::BinaryNotAvailable),
      20, uint(P2PoolManager::ConnectionIssue),
      21, uint(P2PoolManager::HashVerificationFailed),
      22, uint(P2PoolManager::InstallationFailed),

       0        // eod
};

void P2PoolManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<P2PoolManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->p2poolStartFailure(); break;
        case 1: _t->p2poolStatus((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->p2poolDownloadFailure((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->p2poolDownloadSuccess(); break;
        case 4: { bool _r = _t->start((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->exit(); break;
        case 6: { bool _r = _t->isInstalled();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->getStatus(); break;
        case 8: _t->download(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (P2PoolManager::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&P2PoolManager::p2poolStartFailure)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (P2PoolManager::*)(bool , int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&P2PoolManager::p2poolStatus)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (P2PoolManager::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&P2PoolManager::p2poolDownloadFailure)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (P2PoolManager::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&P2PoolManager::p2poolDownloadSuccess)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject P2PoolManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_P2PoolManager.data,
    qt_meta_data_P2PoolManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *P2PoolManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *P2PoolManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_P2PoolManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int P2PoolManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void P2PoolManager::p2poolStartFailure()const
{
    QMetaObject::activate(const_cast< P2PoolManager *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void P2PoolManager::p2poolStatus(bool _t1, int _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< P2PoolManager *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void P2PoolManager::p2poolDownloadFailure(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< P2PoolManager *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void P2PoolManager::p2poolDownloadSuccess()const
{
    QMetaObject::activate(const_cast< P2PoolManager *>(this), &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
