/****************************************************************************
** Meta object code from reading C++ file 'AddressBookModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/model/AddressBookModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AddressBookModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AddressBookModel_t {
    QByteArrayData data[12];
    char stringdata0[183];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AddressBookModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AddressBookModel_t qt_meta_stringdata_AddressBookModel = {
    {
QT_MOC_LITERAL(0, 0, 16), // "AddressBookModel"
QT_MOC_LITERAL(1, 17, 10), // "startReset"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 8), // "endReset"
QT_MOC_LITERAL(4, 38, 9), // "deleteRow"
QT_MOC_LITERAL(5, 48, 3), // "row"
QT_MOC_LITERAL(6, 52, 18), // "AddressBookRowRole"
QT_MOC_LITERAL(7, 71, 15), // "AddressBookRole"
QT_MOC_LITERAL(8, 87, 22), // "AddressBookAddressRole"
QT_MOC_LITERAL(9, 110, 26), // "AddressBookDescriptionRole"
QT_MOC_LITERAL(10, 137, 24), // "AddressBookPaymentIdRole"
QT_MOC_LITERAL(11, 162, 20) // "AddressBookRowIdRole"

    },
    "AddressBookModel\0startReset\0\0endReset\0"
    "deleteRow\0row\0AddressBookRowRole\0"
    "AddressBookRole\0AddressBookAddressRole\0"
    "AddressBookDescriptionRole\0"
    "AddressBookPaymentIdRole\0AddressBookRowIdRole"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AddressBookModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       1,   34, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x0a /* Public */,
       3,    0,   30,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
       4,    1,   31,    2, 0x02 /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Bool, QMetaType::Int,    5,

 // enums: name, alias, flags, count, data
       6,    6, 0x0,    5,   39,

 // enum data: key, value
       7, uint(AddressBookModel::AddressBookRole),
       8, uint(AddressBookModel::AddressBookAddressRole),
       9, uint(AddressBookModel::AddressBookDescriptionRole),
      10, uint(AddressBookModel::AddressBookPaymentIdRole),
      11, uint(AddressBookModel::AddressBookRowIdRole),

       0        // eod
};

void AddressBookModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressBookModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startReset(); break;
        case 1: _t->endReset(); break;
        case 2: { bool _r = _t->deleteRow((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AddressBookModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_AddressBookModel.data,
    qt_meta_data_AddressBookModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AddressBookModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AddressBookModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AddressBookModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int AddressBookModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
