/****************************************************************************
** Meta object code from reading C++ file 'oshelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/main/oshelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'oshelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OSHelper_t {
    QByteArrayData data[24];
    char stringdata0[322];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OSHelper_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OSHelper_t qt_meta_stringdata_OSHelper = {
    {
QT_MOC_LITERAL(0, 0, 8), // "OSHelper"
QT_MOC_LITERAL(1, 9, 18), // "createDesktopEntry"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 16), // "downloadLocation"
QT_MOC_LITERAL(4, 46, 21), // "grabQrCodesFromScreen"
QT_MOC_LITERAL(5, 68, 14), // "QList<QString>"
QT_MOC_LITERAL(6, 83, 8), // "openFile"
QT_MOC_LITERAL(7, 92, 8), // "filePath"
QT_MOC_LITERAL(8, 101, 20), // "openContainingFolder"
QT_MOC_LITERAL(9, 122, 18), // "openSaveFileDialog"
QT_MOC_LITERAL(10, 141, 5), // "title"
QT_MOC_LITERAL(11, 147, 6), // "folder"
QT_MOC_LITERAL(12, 154, 8), // "filename"
QT_MOC_LITERAL(13, 163, 17), // "temporaryFilename"
QT_MOC_LITERAL(14, 181, 13), // "temporaryPath"
QT_MOC_LITERAL(15, 195, 14), // "randomPassword"
QT_MOC_LITERAL(16, 210, 8), // "numBytes"
QT_MOC_LITERAL(17, 219, 21), // "removeTemporaryWallet"
QT_MOC_LITERAL(18, 241, 10), // "walletName"
QT_MOC_LITERAL(19, 252, 10), // "isCapsLock"
QT_MOC_LITERAL(20, 263, 22), // "getNetworkTypeFromFile"
QT_MOC_LITERAL(21, 286, 8), // "keysPath"
QT_MOC_LITERAL(22, 295, 16), // "openSeedTemplate"
QT_MOC_LITERAL(23, 312, 9) // "installed"

    },
    "OSHelper\0createDesktopEntry\0\0"
    "downloadLocation\0grabQrCodesFromScreen\0"
    "QList<QString>\0openFile\0filePath\0"
    "openContainingFolder\0openSaveFileDialog\0"
    "title\0folder\0filename\0temporaryFilename\0"
    "temporaryPath\0randomPassword\0numBytes\0"
    "removeTemporaryWallet\0walletName\0"
    "isCapsLock\0getNetworkTypeFromFile\0"
    "keysPath\0openSeedTemplate\0installed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OSHelper[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       1,  114, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x02 /* Public */,
       3,    0,   85,    2, 0x02 /* Public */,
       4,    0,   86,    2, 0x02 /* Public */,
       6,    1,   87,    2, 0x02 /* Public */,
       8,    1,   90,    2, 0x02 /* Public */,
       9,    3,   93,    2, 0x02 /* Public */,
      13,    0,  100,    2, 0x02 /* Public */,
      14,    0,  101,    2, 0x02 /* Public */,
      15,    1,  102,    2, 0x02 /* Public */,
      15,    0,  105,    2, 0x22 /* Public | MethodCloned */,
      17,    1,  106,    2, 0x02 /* Public */,
      19,    0,  109,    2, 0x02 /* Public */,
      20,    1,  110,    2, 0x02 /* Public */,
      22,    0,  113,    2, 0x02 /* Public */,

 // methods: parameters
    QMetaType::Void,
    QMetaType::QString,
    0x80000000 | 5,
    QMetaType::Bool, QMetaType::QString,    7,
    QMetaType::Bool, QMetaType::QString,    7,
    QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   10,   11,   12,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString, QMetaType::Int,   16,
    QMetaType::QString,
    QMetaType::Bool, QMetaType::QString,   18,
    QMetaType::Bool,
    QMetaType::UChar, QMetaType::QString,   21,
    QMetaType::Void,

 // properties: name, type, flags
      23, QMetaType::Bool, 0x00095401,

       0        // eod
};

void OSHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OSHelper *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->createDesktopEntry(); break;
        case 1: { QString _r = _t->downloadLocation();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QList<QString> _r = _t->grabQrCodesFromScreen();
            if (_a[0]) *reinterpret_cast< QList<QString>*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->openFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->openContainingFolder((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->openSaveFileDialog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->temporaryFilename();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->temporaryPath();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->randomPassword((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->randomPassword();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->removeTemporaryWallet((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->isCapsLock();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { quint8 _r = _t->getNetworkTypeFromFile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint8*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->openSeedTemplate(); break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<OSHelper *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->installed(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject OSHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_OSHelper.data,
    qt_meta_data_OSHelper,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OSHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OSHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OSHelper.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OSHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
