/****************************************************************************
** Meta object code from reading C++ file 'TransactionHistoryModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/model/TransactionHistoryModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TransactionHistoryModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TransactionHistoryModel_t {
    QByteArrayData data[26];
    char stringdata0[609];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TransactionHistoryModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TransactionHistoryModel_t qt_meta_stringdata_TransactionHistoryModel = {
    {
QT_MOC_LITERAL(0, 0, 23), // "TransactionHistoryModel"
QT_MOC_LITERAL(1, 24, 25), // "transactionHistoryChanged"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 18), // "transactionHistory"
QT_MOC_LITERAL(4, 70, 19), // "TransactionHistory*"
QT_MOC_LITERAL(5, 90, 19), // "TransactionInfoRole"
QT_MOC_LITERAL(6, 110, 24), // "TransactionDirectionRole"
QT_MOC_LITERAL(7, 135, 22), // "TransactionPendingRole"
QT_MOC_LITERAL(8, 158, 21), // "TransactionFailedRole"
QT_MOC_LITERAL(9, 180, 21), // "TransactionAmountRole"
QT_MOC_LITERAL(10, 202, 28), // "TransactionDisplayAmountRole"
QT_MOC_LITERAL(11, 231, 18), // "TransactionFeeRole"
QT_MOC_LITERAL(12, 250, 26), // "TransactionBlockHeightRole"
QT_MOC_LITERAL(13, 277, 27), // "TransactionSubaddrIndexRole"
QT_MOC_LITERAL(14, 305, 29), // "TransactionSubaddrAccountRole"
QT_MOC_LITERAL(15, 335, 20), // "TransactionLabelRole"
QT_MOC_LITERAL(16, 356, 28), // "TransactionConfirmationsRole"
QT_MOC_LITERAL(17, 385, 36), // "TransactionConfirmationsRequi..."
QT_MOC_LITERAL(18, 422, 19), // "TransactionHashRole"
QT_MOC_LITERAL(19, 442, 24), // "TransactionTimeStampRole"
QT_MOC_LITERAL(20, 467, 24), // "TransactionPaymentIdRole"
QT_MOC_LITERAL(21, 492, 20), // "TransactionIsOutRole"
QT_MOC_LITERAL(22, 513, 19), // "TransactionDateRole"
QT_MOC_LITERAL(23, 533, 19), // "TransactionTimeRole"
QT_MOC_LITERAL(24, 553, 27), // "TransactionAtomicAmountRole"
QT_MOC_LITERAL(25, 581, 27) // "TransactionDestinationsRole"

    },
    "TransactionHistoryModel\0"
    "transactionHistoryChanged\0\0"
    "transactionHistory\0TransactionHistory*\0"
    "TransactionInfoRole\0TransactionDirectionRole\0"
    "TransactionPendingRole\0TransactionFailedRole\0"
    "TransactionAmountRole\0"
    "TransactionDisplayAmountRole\0"
    "TransactionFeeRole\0TransactionBlockHeightRole\0"
    "TransactionSubaddrIndexRole\0"
    "TransactionSubaddrAccountRole\0"
    "TransactionLabelRole\0TransactionConfirmationsRole\0"
    "TransactionConfirmationsRequiredRole\0"
    "TransactionHashRole\0TransactionTimeStampRole\0"
    "TransactionPaymentIdRole\0TransactionIsOutRole\0"
    "TransactionDateRole\0TransactionTimeRole\0"
    "TransactionAtomicAmountRole\0"
    "TransactionDestinationsRole"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TransactionHistoryModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   20, // properties
       1,   24, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       3, 0x80000000 | 4, 0x0049510b,

 // properties: notify_signal_id
       0,

 // enums: name, alias, flags, count, data
       5,    5, 0x0,   20,   29,

 // enum data: key, value
       6, uint(TransactionHistoryModel::TransactionDirectionRole),
       7, uint(TransactionHistoryModel::TransactionPendingRole),
       8, uint(TransactionHistoryModel::TransactionFailedRole),
       9, uint(TransactionHistoryModel::TransactionAmountRole),
      10, uint(TransactionHistoryModel::TransactionDisplayAmountRole),
      11, uint(TransactionHistoryModel::TransactionFeeRole),
      12, uint(TransactionHistoryModel::TransactionBlockHeightRole),
      13, uint(TransactionHistoryModel::TransactionSubaddrIndexRole),
      14, uint(TransactionHistoryModel::TransactionSubaddrAccountRole),
      15, uint(TransactionHistoryModel::TransactionLabelRole),
      16, uint(TransactionHistoryModel::TransactionConfirmationsRole),
      17, uint(TransactionHistoryModel::TransactionConfirmationsRequiredRole),
      18, uint(TransactionHistoryModel::TransactionHashRole),
      19, uint(TransactionHistoryModel::TransactionTimeStampRole),
      20, uint(TransactionHistoryModel::TransactionPaymentIdRole),
      21, uint(TransactionHistoryModel::TransactionIsOutRole),
      22, uint(TransactionHistoryModel::TransactionDateRole),
      23, uint(TransactionHistoryModel::TransactionTimeRole),
      24, uint(TransactionHistoryModel::TransactionAtomicAmountRole),
      25, uint(TransactionHistoryModel::TransactionDestinationsRole),

       0        // eod
};

void TransactionHistoryModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TransactionHistoryModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->transactionHistoryChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TransactionHistoryModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransactionHistoryModel::transactionHistoryChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TransactionHistoryModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< TransactionHistory**>(_v) = _t->transactionHistory(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TransactionHistoryModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTransactionHistory(*reinterpret_cast< TransactionHistory**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TransactionHistoryModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_TransactionHistoryModel.data,
    qt_meta_data_TransactionHistoryModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TransactionHistoryModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TransactionHistoryModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TransactionHistoryModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int TransactionHistoryModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TransactionHistoryModel::transactionHistoryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
